/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.banking.dtaus.swing;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFormattedTextField;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import org.jdtaus.banking.AlphaNumericText27;
import org.jdtaus.banking.Bankleitzahl;
import org.jdtaus.banking.Kontonummer;
import org.jdtaus.banking.Textschluessel;
import org.jdtaus.banking.TextschluesselVerzeichnis;
import org.jdtaus.banking.dtaus.LogicalFileType;
import org.jdtaus.banking.swing.AlphaNumericText27TextField;
import org.jdtaus.banking.swing.BankleitzahlTextField;
import org.jdtaus.banking.swing.KontonummerTextField;
import org.jdtaus.banking.swing.TextschluesselRenderer;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ContextInitializer;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Properties;
import org.jdtaus.core.container.Property;
import org.jdtaus.core.text.Message;
import org.jdtaus.editor.GeneratorTask;
import org.jdtaus.editor.IconSet;
import org.jdtaus.editor.IllegalGeneratorTaskException;

/**
 * Dialog for generating a DTAUS test file.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: GeneratorDialog.java 3512 2007-05-16 05:57:21Z schulte2005 $
 */
public class GeneratorDialog extends JDialog implements ContainerInitializer
{
    //--Implementation----------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(GeneratorDialog.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /**
     * Initializes the properties of the instance.
     *
     * @param meta the property values to initialize the instance with.
     *
     * @throws NullPointerException if {@code meta} is {@code null}.
     */
    protected void initializeProperties(final Properties meta)
    {
        Property p;

        if(meta == null)
        {
            throw new NullPointerException("meta");
        }

    }

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    /** Initializes the instance centering the dialog on the screen. */
    public void initialize()
    {
        final Dimension screenSize = Toolkit.getDefaultToolkit().
            getScreenSize();

        this.initComponents();

        this.setLocation(screenSize.width / 2 - this.getWidth() / 2,
            screenSize.height / 2 - this.getHeight() / 2);

    }

    //----------------------------------------------------ContainerInitializer--
    //--Dependencies------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Configured <code>UserInterface</code> implementation. */
    private transient EditorGUI _dependency2;

    /**
     * Gets the configured <code>UserInterface</code> implementation.
     *
     * @return the configured <code>UserInterface</code> implementation.
     */
    private EditorGUI getUserInterface()
    {
        EditorGUI ret = null;
        if(this._dependency2 != null)
        {
            ret = this._dependency2;
        }
        else
        {
            ret = (EditorGUI) ContainerFactory.getContainer().
                getDependency(GeneratorDialog.class,
                "UserInterface");

            if(ModelFactory.getModel().getModules().
                getImplementation(GeneratorDialog.class.getName()).
                getDependencies().getDependency("UserInterface").
                isBound())
            {
                this._dependency2 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>IconSet</code> implementation. */
    private transient IconSet _dependency1;

    /**
     * Gets the configured <code>IconSet</code> implementation.
     *
     * @return the configured <code>IconSet</code> implementation.
     */
    private IconSet getIconSet()
    {
        IconSet ret = null;
        if(this._dependency1 != null)
        {
            ret = this._dependency1;
        }
        else
        {
            ret = (IconSet) ContainerFactory.getContainer().
                getDependency(GeneratorDialog.class,
                "IconSet");

            if(ModelFactory.getModel().getModules().
                getImplementation(GeneratorDialog.class.getName()).
                getDependencies().getDependency("IconSet").
                isBound())
            {
                this._dependency1 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>TextschluesselVerzeichnis</code> implementation. */
    private transient TextschluesselVerzeichnis _dependency0;

    /**
     * Gets the configured <code>TextschluesselVerzeichnis</code> implementation.
     *
     * @return the configured <code>TextschluesselVerzeichnis</code> implementation.
     */
    private TextschluesselVerzeichnis getTextschluesselVerzeichnis()
    {
        TextschluesselVerzeichnis ret = null;
        if(this._dependency0 != null)
        {
            ret = this._dependency0;
        }
        else
        {
            ret = (TextschluesselVerzeichnis) ContainerFactory.getContainer().
                getDependency(GeneratorDialog.class,
                "TextschluesselVerzeichnis");

            if(ModelFactory.getModel().getModules().
                getImplementation(GeneratorDialog.class.getName()).
                getDependencies().getDependency("TextschluesselVerzeichnis").
                isBound())
            {
                this._dependency0 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }

    //------------------------------------------------------------Dependencies--
    //--Properties--------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.


    //--------------------------------------------------------------Properties--
    //--GeneratorDialog---------------------------------------------------------

    /** A return status code - returned if Cancel button has been pressed */
    public static final int RET_CANCEL = 0;

    /** A return status code - returned if OK button has been pressed */
    public static final int RET_OK = 1;

    /**
     * Creates new {@code GeneratorDialog} instance.
     *
     * @param owner the frame owning the dialog.
     * @param modal {@code true} for a modal dialog; {@code false} else.
     */
    public GeneratorDialog(final Frame owner, final boolean modal)
    {
        super(owner, modal);
        this.initializeProperties(GeneratorDialog.META.getProperties());
        this.initialize();
    }

    /**
     * Gets the return status of the dialog.
     *
     * @return the return status of this dialog - one of {@code RET_OK} or
     * {@code RET_CANCEL}.
     */
    public int getReturnStatus()
    {
        return this.returnStatus;
    }

    /**
     * Gets the output file.
     *
     * @return the output file or {@code null} if no file has been specified.
     */
    public File getOutputFile()
    {
        return this.outputFile;
    }

    /**
     * Sets the output file.
     *
     * @param value the new output file or {@code null}.
     */
    public void setOutputFile(final File file)
    {
        this.firePropertyChange("outputFile", this.outputFile, file);
        this.outputFile = file;
    }

    /**
     * Gets the {@code ActionListener} called when the user requests this
     * dialog's operation.
     *
     * @return the {@code ActionListener} called when the user requests this
     * dialog's operation or {@code null}.
     */
    public ActionListener getActionListener()
    {
        return this.actionListener;
    }

    /**
     * Sets the {@code ActionListener} to be called when the user requests this
     * dialog's operation.
     *
     * @param listener the {@code ActionListener} to be called when the user
     * requests this dialog's operation or {@code null}.
     */
    public void setActionListener(final ActionListener listener)
    {
        this.actionListener = listener;
    }

    /**
     * Gets the task data displayed by the dialog.
     *
     * @return the task data displayed by the dialog.
     */
    public GeneratorTask getGeneratorTask()
    {
        final GeneratorTask task = new GeneratorTask();
        task.setCustomerAccount(
            ((KontonummerTextField) this.headerAccountTextField).
            getKontonummer());

        task.setCustomerBank(((BankleitzahlTextField) this.headerBankTextField).
            getBankleitzahl());

        task.setCustomerName(
            ((AlphaNumericText27TextField) this.headerCustomerTextField).
            getAlphaNumericText27());

        if(this.randomDescriptionCheckBox.isSelected())
        {
            task.setDescriptionCount(GeneratorTask.RANDOM);
        }
        else
        {
            task.setDescriptionCount(
                (Number) ((JFormattedTextField) this.numDescriptionsTextField).
                getValue());

        }

        task.setExecutiveAccount(
            ((KontonummerTextField) this.executiveAccountTextField).
            getKontonummer());

        task.setExecutiveBank(
            ((BankleitzahlTextField) this.executiveBankTextField).
            getBankleitzahl());

        task.setExecutiveName(
            ((AlphaNumericText27TextField) this.executiveNameTextField).
            getAlphaNumericText27());

        task.setFileCount(
            (Number) ((JFormattedTextField) this.numFilesTextField).
            getValue());

        if(this.fileTypeList.getSelectedValues() != null)
        {
            final Object[] selection = this.fileTypeList.getSelectedValues();
            final List<LogicalFileType> types = new ArrayList(selection.length);
            for(int i = selection.length - 1; i >= 0; i--)
            {
                if(selection[i] instanceof LogicalFileType)
                {
                    types.add((LogicalFileType) selection[i]);
                }
            }

            task.setFileTypes((LogicalFileType[]) types.
                toArray(new LogicalFileType[types.size()]));

        }

        task.setTargetAccount(
            ((KontonummerTextField) this.targetAccountTextField).
            getKontonummer());

        task.setTargetBank(
            ((BankleitzahlTextField) this.targetBankTextField).
            getBankleitzahl());

        task.setTargetName(
            ((AlphaNumericText27TextField) this.targetNameTextField).
            getAlphaNumericText27());

        if(this.textschluesselList.getSelectedValues() != null)
        {
            final Object[] selection = this.textschluesselList.
                getSelectedValues();

            final List<Textschluessel> keys = new ArrayList(selection.length);
            for(int i = selection.length - 1; i >= 0; i--)
            {
                if(selection[i] instanceof Textschluessel)
                {
                    keys.add((Textschluessel) selection[i]);
                }
            }

            task.setTextschluessel((Textschluessel[]) keys.toArray(
                new Textschluessel[keys.size()]));

        }

        task.setTransactionCount(
            (Number) ((JFormattedTextField) this.numTransactionsTextField).
            getValue());

        return task;
    }

    /**
     * Sets the task data to be displayed by the dialog.
     *
     * @param task the task data to be displayed by the dialog or
     * {@code null} to reset the dialog display.
     */
    public void setGeneratorTask(final GeneratorTask task)
    {
        if(task == null)
        {
            ((JFormattedTextField) this.executiveAccountTextField).
                setValue(null);

            ((JFormattedTextField) this.executiveBankTextField).
                setValue(null);

            ((JFormattedTextField) this.executiveNameTextField).
                setValue(null);

            ((JFormattedTextField) this.targetAccountTextField).
                setValue(null);

            ((JFormattedTextField) this.targetBankTextField).
                setValue(null);

            ((JFormattedTextField) this.targetNameTextField).
                setValue(null);

            ((JFormattedTextField) this.headerAccountTextField).
                setValue(null);

            ((JFormattedTextField) this.headerBankTextField).
                setValue(null);

            ((JFormattedTextField) this.headerCustomerTextField).
                setValue(null);

            ((JFormattedTextField) this.numDescriptionsTextField).
                setValue(null);

            ((JFormattedTextField) this.numFilesTextField).
                setValue(null);

            ((JFormattedTextField) this.numTransactionsTextField).
                setValue(null);

            this.fileTypeList.getSelectionModel().clearSelection();
            this.textschluesselList.getSelectionModel().clearSelection();
            this.randomDescriptionCheckBox.setSelected(false);
        }
        else
        {
            ((JFormattedTextField) this.executiveAccountTextField).
                setValue(task.getExecutiveAccount());

            ((JFormattedTextField) this.executiveBankTextField).
                setValue(task.getExecutiveBank());

            ((JFormattedTextField) this.executiveNameTextField).
                setValue(task.getExecutiveName());

            ((JFormattedTextField) this.targetAccountTextField).
                setValue(task.getTargetAccount());

            ((JFormattedTextField) this.targetBankTextField).
                setValue(task.getTargetBank());

            ((JFormattedTextField) this.targetNameTextField).
                setValue(task.getTargetName());

            ((JFormattedTextField) this.headerAccountTextField).
                setValue(task.getCustomerAccount());

            ((JFormattedTextField) this.headerBankTextField).
                setValue(task.getCustomerBank());

            ((JFormattedTextField) this.headerCustomerTextField).
                setValue(task.getCustomerName());

            if(task.getDescriptionCount() == null ||
                !task.getDescriptionCount().equals(GeneratorTask.RANDOM))
            {
                ((JFormattedTextField) this.numDescriptionsTextField).
                    setValue(task.getDescriptionCount());

            }

            ((JFormattedTextField) this.numFilesTextField).
                setValue(task.getFileCount());

            ((JFormattedTextField) this.numTransactionsTextField).
                setValue(task.getTransactionCount());

            this.randomDescriptionCheckBox.setSelected(
                task.getDescriptionCount().equals(GeneratorTask.RANDOM));

            if(task.getFileTypes() == null)
            {
                this.fileTypeList.getSelectionModel().clearSelection();
            }
            else
            {
                final List<Integer> indexList =
                    new ArrayList(task.getFileTypes().length);

                for(int i = this.fileTypeList.getModel().
                    getSize() - 1; i >= 0; i--)
                {
                    final LogicalFileType type =
                        (LogicalFileType) this.fileTypeList.getModel().
                        getElementAt(i);

                    for(int j = task.getFileTypes().length - 1; j >= 0; j--)
                    {
                        if(type.equals(task.getFileTypes()[j]))
                        {
                            indexList.add(Integer.valueOf(i));
                            break;
                        }
                    }
                }

                final int[] indices = new int[indexList.size()];
                for(int i = indexList.size() - 1; i >= 0; i--)
                {
                    indices[i] = indexList.get(i);
                }

                this.fileTypeList.setSelectedIndices(indices);
            }

            if(task.getTextschluessel() == null)
            {
                this.textschluesselList.getSelectionModel().clearSelection();
            }
            else
            {
                final List<Integer> indexList =
                    new ArrayList(task.getTextschluessel().length);

                for(int i = this.textschluesselList.getModel().
                    getSize() - 1; i >= 0; i--)
                {
                    final Textschluessel key =
                        (Textschluessel) this.textschluesselList.getModel().
                        getElementAt(i);

                    for(int j = task.getTextschluessel().
                        length - 1; j >= 0; j--)
                    {
                        if(key.equals(task.getTextschluessel()[j]))
                        {
                            indexList.add(Integer.valueOf(i));
                            break;
                        }
                    }
                }

                final int[] indices = new int[indexList.size()];
                for(int i = indexList.size() - 1; i >= 0; i--)
                {
                    indices[i] = indexList.get(i);
                }

                this.textschluesselList.setSelectedIndices(indices);
            }
        }
    }

    /**
     * Updates the dialog according to a caught
     * {@code IllegalGeneratorTaskException}.
     *
     * @param e the caught {@code IllegalGeneratorTaskException} to update the
     * dialog with or {@code null} to reset the dialog from any caught
     * {@code IllegalGeneratorTaskException}s.
     */
    public void onIllegalGeneratorTask(final IllegalGeneratorTaskException e)
    {
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                if(e == null)
                {
                    for(JComponent comp : colors.keySet())
                    {
                        resetForeground(comp);
                    };

                    executiveAccountTextField.setToolTipText(null);
                    executiveBankTextField.setToolTipText(null);
                    executiveNameTextField.setToolTipText(null);
                    targetAccountTextField.setToolTipText(null);
                    targetBankTextField.setToolTipText(null);
                    targetNameTextField.setToolTipText(null);
                    headerAccountTextField.setToolTipText(null);
                    headerBankTextField.setToolTipText(null);
                    headerCustomerTextField.setToolTipText(null);
                    numDescriptionsTextField.setToolTipText(null);
                    numFilesTextField.setToolTipText(null);
                    numTransactionsTextField.setToolTipText(null);
                    fileTypeList.setToolTipText(null);
                    textschluesselList.setToolTipText(null);
                }
                else
                {
                    handleIllegalGeneratorTask(executiveAccountLabel,
                        executiveAccountTextField,
                        e.getMessages(GeneratorTask.PROP_EXECUTIVEACCOUNT));

                    handleIllegalGeneratorTask(executiveBankLabel,
                        executiveBankTextField,
                        e.getMessages(GeneratorTask.PROP_EXECUTIVEBANK));

                    handleIllegalGeneratorTask(executiveNameLabel,
                        executiveNameTextField,
                        e.getMessages(GeneratorTask.PROP_EXECUTIVENAME));

                    handleIllegalGeneratorTask(targetAccountLabel,
                        targetAccountTextField,
                        e.getMessages(GeneratorTask.PROP_TARGETACCOUNT));

                    handleIllegalGeneratorTask(targetBankLabel,
                        targetBankTextField,
                        e.getMessages(GeneratorTask.PROP_TARGETBANK));

                    handleIllegalGeneratorTask(targetNameLabel,
                        targetNameTextField,
                        e.getMessages(GeneratorTask.PROP_TARGETNAME));

                    handleIllegalGeneratorTask(headerAccountLabel,
                        headerAccountTextField,
                        e.getMessages(GeneratorTask.PROP_CUSTOMERACCOUNT));

                    handleIllegalGeneratorTask(headerBankLabel,
                        headerBankTextField,
                        e.getMessages(GeneratorTask.PROP_CUSTOMERBANK));

                    handleIllegalGeneratorTask(headerCustomerLabel,
                        headerCustomerTextField,
                        e.getMessages(GeneratorTask.PROP_CUSTOMERNAME));

                    handleIllegalGeneratorTask(numDescriptionsLabel,
                        numDescriptionsTextField,
                        e.getMessages(GeneratorTask.PROP_DESCRIPTIONCOUNT));

                    handleIllegalGeneratorTask(numFilesLabel,
                        numFilesTextField,
                        e.getMessages(GeneratorTask.PROP_FILECOUNT));

                    handleIllegalGeneratorTask(numTransactionsLabel,
                        numTransactionsTextField,
                        e.getMessages(GeneratorTask.PROP_TRANSACTIONCOUNT));

                    handleIllegalGeneratorTask(fileTypeList, fileTypeList,
                        e.getMessages(GeneratorTask.PROP_FILETYPES));

                    handleIllegalGeneratorTask(textschluesselList,
                        textschluesselList,
                        e.getMessages(GeneratorTask.PROP_TEXTSCHLUESSEL));

                    final Message[] msgs = e.getMessages();
                    if(msgs.length > 0)
                    {
                        final StringBuffer buf =
                            new StringBuffer(msgs.length * 150);

                        for(int i = 0; i < msgs.length; i++)
                        {
                            buf.append(msgs[i].getText(Locale.getDefault())).
                                append('\n');

                        }

                        JOptionPane.showMessageDialog(getUserInterface().
                            getParent(), buf.toString(), e.getMessage(),
                            JOptionPane.INFORMATION_MESSAGE);

                    }
                }
            }
        });
    }

    /**
     * Flag indicating if the dialog is locked for any interaction.
     *
     * @return {@code true} if the dialog is locked for any interaction;
     * {@code false} if the dialog is not locked.
     */
    public boolean isLocked()
    {
        return this.locked;
    }

    /**
     * Setter for property {@code locked}.
     *
     * @param locked {@code true} if the dialog should be locked for any
     * interaction; {@code false} to unlock the dialog.
     */
    public void setLocked(final boolean locked)
    {
        this.setCursor(Cursor.getPredefinedCursor(locked ?
            Cursor.WAIT_CURSOR : Cursor.DEFAULT_CURSOR));

        this.closeButton.setEnabled(!locked);
        this.executiveAccountTextField.setEnabled(!locked);
        this.executiveBankTextField.setEnabled(!locked);
        this.executiveNameTextField.setEnabled(!locked);
        this.fileTypeList.setEnabled(!locked);
        this.generateButton.setEnabled(!locked);
        this.headerAccountTextField.setEnabled(!locked);
        this.headerBankTextField.setEnabled(!locked);
        this.headerCustomerTextField.setEnabled(!locked);
        this.numDescriptionsTextField.setEnabled(!locked);
        this.numFilesTextField.setEnabled(!locked);
        this.numTransactionsTextField.setEnabled(!locked);
        this.randomDescriptionCheckBox.setEnabled(!locked);
        this.searchExecutiveBankleitzahlButton.setEnabled(!locked);
        this.searchHeaderBankleitzahlButton.setEnabled(!locked);
        this.searchTargetBankleitzahlButton.setEnabled(!locked);
        this.targetAccountTextField.setEnabled(!locked);
        this.targetBankTextField.setEnabled(!locked);
        this.targetNameTextField.setEnabled(!locked);
        this.textschluesselList.setEnabled(!locked);
        this.selectedFileTextField.setEnabled(!locked);
        this.chooseFileButton.setEnabled(!locked);

        this.firePropertyChange("locked", this.locked, locked);
        this.locked = locked;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        generateButton = new javax.swing.JButton();
        closeButton = new javax.swing.JButton();
        executivePanel = new javax.swing.JPanel();
        executiveNameLabel = new javax.swing.JLabel();
        executiveNameTextField = new AlphaNumericText27TextField();
        executiveBankLabel = new javax.swing.JLabel();
        executiveBankTextField = new BankleitzahlTextField();
        executiveAccountLabel = new javax.swing.JLabel();
        executiveAccountTextField = new KontonummerTextField();
        searchExecutiveBankleitzahlButton = new javax.swing.JButton();
        targetPanel = new javax.swing.JPanel();
        targetNameLabel = new javax.swing.JLabel();
        targetNameTextField = new AlphaNumericText27TextField();
        targetBankLabel = new javax.swing.JLabel();
        targetBankTextField = new BankleitzahlTextField();
        targetAccountLabel = new javax.swing.JLabel();
        targetAccountTextField = new KontonummerTextField();
        searchTargetBankleitzahlButton = new javax.swing.JButton();
        textschluesselPanel = new javax.swing.JPanel();
        textschluesselScrollPane = new javax.swing.JScrollPane();
        textschluesselList = new javax.swing.JList();
        fileTypePanel = new javax.swing.JPanel();
        fileTypeScrollPane = new javax.swing.JScrollPane();
        fileTypeList = new javax.swing.JList();
        detailsPanel = new javax.swing.JPanel();
        numFilesLabel = new javax.swing.JLabel();
        numFilesTextField = new javax.swing.JFormattedTextField();
        numTransactionsLabel = new javax.swing.JLabel();
        numTransactionsTextField = new javax.swing.JFormattedTextField();
        numDescriptionsLabel = new javax.swing.JLabel();
        numDescriptionsTextField = new javax.swing.JFormattedTextField();
        randomDescriptionCheckBox = new javax.swing.JCheckBox();
        headerCustomerLabel = new javax.swing.JLabel();
        headerCustomerTextField = new AlphaNumericText27TextField();
        headerBankLabel = new javax.swing.JLabel();
        headerBankTextField = new BankleitzahlTextField();
        headerAccountLabel = new javax.swing.JLabel();
        headerAccountTextField = new KontonummerTextField();
        searchHeaderBankleitzahlButton = new javax.swing.JButton();
        selectedFileLabel = new javax.swing.JLabel();
        selectedFileTextField = new javax.swing.JTextField();
        chooseFileButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle(GeneratorDialogBundle.getTitleText(Locale.getDefault()));
        setName("Generator Dialog");
        addPropertyChangeListener(new java.beans.PropertyChangeListener()
        {
            public void propertyChange(java.beans.PropertyChangeEvent evt)
            {
                formPropertyChange(evt);
            }
        });
        addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(java.awt.event.WindowEvent evt)
            {
                closeDialog(evt);
            }
        });

        generateButton.setText(GeneratorDialogBundle.getGenerateButtonText(Locale.getDefault()));
        generateButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                generateButtonActionPerformed(evt);
            }
        });

        closeButton.setText(GeneratorDialogBundle.getCloseButtonText(Locale.getDefault()));
        closeButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                closeButtonActionPerformed(evt);
            }
        });

        executivePanel.setBorder(javax.swing.BorderFactory.createTitledBorder(GeneratorDialogBundle.getExecutivePanelTitleText(Locale.getDefault())));
        executiveNameLabel.setText(GeneratorDialogBundle.getExecutiveNameText(Locale.getDefault()));

        executiveNameTextField.setColumns(AlphaNumericText27.MAX_LENGTH);

        executiveBankLabel.setText(GeneratorDialogBundle.getExecutiveBankText(Locale.getDefault()));

        executiveBankTextField.setColumns(Bankleitzahl.MAX_CHARACTERS);

        executiveAccountLabel.setText(GeneratorDialogBundle.getExecutiveAccountText(Locale.getDefault()));

        executiveAccountTextField.setColumns(Kontonummer.MAX_CHARACTERS);

        searchExecutiveBankleitzahlButton.setAction(new SearchBankleitzahlAction(this.executiveBankTextField, true));
        searchExecutiveBankleitzahlButton.setBorderPainted(false);
        searchExecutiveBankleitzahlButton.setContentAreaFilled(false);
        searchExecutiveBankleitzahlButton.setDefaultCapable(false);
        searchExecutiveBankleitzahlButton.setMaximumSize(new java.awt.Dimension(16, 16));
        searchExecutiveBankleitzahlButton.setMinimumSize(new java.awt.Dimension(16, 16));
        searchExecutiveBankleitzahlButton.setPreferredSize(new java.awt.Dimension(16, 16));

        org.jdesktop.layout.GroupLayout executivePanelLayout = new org.jdesktop.layout.GroupLayout(executivePanel);
        executivePanel.setLayout(executivePanelLayout);
        executivePanelLayout.setHorizontalGroup(
            executivePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(executivePanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(executivePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(executiveAccountLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(executiveNameLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(executiveBankLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(executivePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(executiveAccountTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 289, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, executivePanelLayout.createSequentialGroup()
                        .add(executiveBankTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 267, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(searchExecutiveBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, executiveNameTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 289, Short.MAX_VALUE))
                .addContainerGap())
        );
        executivePanelLayout.setVerticalGroup(
            executivePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(executivePanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(executivePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(executiveNameLabel)
                    .add(executiveNameTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(executivePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(searchExecutiveBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(executiveBankTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(executiveBankLabel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 17, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(executivePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(executiveAccountTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(executiveAccountLabel))
                .addContainerGap(36, Short.MAX_VALUE))
        );

        targetPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(GeneratorDialogBundle.getTargetPanelTitleText(Locale.getDefault())));
        targetNameLabel.setText(GeneratorDialogBundle.getTargetNameText(Locale.getDefault()));

        targetNameTextField.setColumns(AlphaNumericText27.MAX_LENGTH);

        targetBankLabel.setText(GeneratorDialogBundle.getTargetBankText(Locale.getDefault()));

        targetBankTextField.setColumns(Bankleitzahl.MAX_CHARACTERS);

        targetAccountLabel.setText(GeneratorDialogBundle.getTargetAccountText(Locale.getDefault()));

        targetAccountTextField.setColumns(Kontonummer.MAX_CHARACTERS);

        searchTargetBankleitzahlButton.setAction(new SearchBankleitzahlAction(this.targetBankTextField, true));
        searchTargetBankleitzahlButton.setBorderPainted(false);
        searchTargetBankleitzahlButton.setContentAreaFilled(false);
        searchTargetBankleitzahlButton.setDefaultCapable(false);
        searchTargetBankleitzahlButton.setMaximumSize(new java.awt.Dimension(16, 16));
        searchTargetBankleitzahlButton.setMinimumSize(new java.awt.Dimension(16, 16));
        searchTargetBankleitzahlButton.setPreferredSize(new java.awt.Dimension(16, 16));

        org.jdesktop.layout.GroupLayout targetPanelLayout = new org.jdesktop.layout.GroupLayout(targetPanel);
        targetPanel.setLayout(targetPanelLayout);
        targetPanelLayout.setHorizontalGroup(
            targetPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(targetPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(targetPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(targetAccountLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(targetBankLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(targetNameLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(targetPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, targetPanelLayout.createSequentialGroup()
                        .add(targetBankTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 268, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(searchTargetBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(targetNameTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 290, Short.MAX_VALUE)
                    .add(targetAccountTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 290, Short.MAX_VALUE))
                .addContainerGap())
        );
        targetPanelLayout.setVerticalGroup(
            targetPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(targetPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(targetPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(targetNameLabel)
                    .add(targetNameTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(targetPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(searchTargetBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(targetBankLabel)
                    .add(targetBankTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(targetPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(targetAccountLabel)
                    .add(targetAccountTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(36, Short.MAX_VALUE))
        );

        textschluesselPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(GeneratorDialogBundle.getTextschluesselPanelTitleText(Locale.getDefault())));
        textschluesselList.setModel(new DefaultListModel());
        textschluesselList.setCellRenderer(new TextschluesselRenderer());
        for(Textschluessel key : this.getSupportedTextschluessel())
        {
            ((DefaultListModel) this.textschluesselList.getModel()).addElement(key);
        }
        textschluesselScrollPane.setViewportView(textschluesselList);

        org.jdesktop.layout.GroupLayout textschluesselPanelLayout = new org.jdesktop.layout.GroupLayout(textschluesselPanel);
        textschluesselPanel.setLayout(textschluesselPanelLayout);
        textschluesselPanelLayout.setHorizontalGroup(
            textschluesselPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, textschluesselScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 410, Short.MAX_VALUE)
        );
        textschluesselPanelLayout.setVerticalGroup(
            textschluesselPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(textschluesselScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 113, Short.MAX_VALUE)
        );

        fileTypePanel.setBorder(javax.swing.BorderFactory.createTitledBorder(GeneratorDialogBundle.getFileTypePanelTitleText(Locale.getDefault())));
        fileTypeList.setModel(new DefaultListModel());
        fileTypeList.setCellRenderer(new LogicalFileTypeRenderer());
        for(LogicalFileType type : this.getSupportedFileTypes())
        {
            ((DefaultListModel) this.fileTypeList.getModel()).addElement(type);
        }
        fileTypeList.addListSelectionListener(new javax.swing.event.ListSelectionListener()
        {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt)
            {
                fileTypeListValueChanged(evt);
            }
        });

        fileTypeScrollPane.setViewportView(fileTypeList);

        org.jdesktop.layout.GroupLayout fileTypePanelLayout = new org.jdesktop.layout.GroupLayout(fileTypePanel);
        fileTypePanel.setLayout(fileTypePanelLayout);
        fileTypePanelLayout.setHorizontalGroup(
            fileTypePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(fileTypeScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 409, Short.MAX_VALUE)
        );
        fileTypePanelLayout.setVerticalGroup(
            fileTypePanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(fileTypeScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 113, Short.MAX_VALUE)
        );

        detailsPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(GeneratorDialogBundle.getDetailsPanelTitleText(Locale.getDefault())));
        numFilesLabel.setText(GeneratorDialogBundle.getNumFilesText(Locale.getDefault()));

        numFilesTextField.setColumns(20);
        numFilesTextField.setValue(new Long(0L));

        numTransactionsLabel.setText(GeneratorDialogBundle.getNumTransactionsText(Locale.getDefault()));

        numTransactionsTextField.setColumns(7);
        numTransactionsTextField.setValue(new Integer(0));

        numDescriptionsLabel.setText(GeneratorDialogBundle.getNumDescriptionsText(Locale.getDefault()));

        numDescriptionsTextField.setColumns(2);
        numDescriptionsTextField.setValue(new Integer(0));

        randomDescriptionCheckBox.setText(GeneratorDialogBundle.getVariableDescriptionText(Locale.getDefault()));
        randomDescriptionCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        randomDescriptionCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        randomDescriptionCheckBox.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                randomDescriptionCheckBoxActionPerformed(evt);
                randomDescriptionCheckBoxActionPerformed1(evt);
            }
        });

        headerCustomerLabel.setText(GeneratorDialogBundle.getHeaderCustomerText(Locale.getDefault()));

        headerCustomerTextField.setColumns(AlphaNumericText27.MAX_LENGTH);

        headerBankLabel.setText(GeneratorDialogBundle.getHeaderBankText(Locale.getDefault()));

        headerBankTextField.setColumns(Bankleitzahl.MAX_CHARACTERS);

        headerAccountLabel.setText(GeneratorDialogBundle.getHeaderAccountText(Locale.getDefault()));

        headerAccountTextField.setColumns(Kontonummer.MAX_CHARACTERS);

        searchHeaderBankleitzahlButton.setAction(new SearchBankleitzahlAction(this.headerBankTextField, true));
        searchHeaderBankleitzahlButton.setBorderPainted(false);
        searchHeaderBankleitzahlButton.setContentAreaFilled(false);
        searchHeaderBankleitzahlButton.setDefaultCapable(false);
        searchHeaderBankleitzahlButton.setMaximumSize(new java.awt.Dimension(16, 16));
        searchHeaderBankleitzahlButton.setMinimumSize(new java.awt.Dimension(16, 16));
        searchHeaderBankleitzahlButton.setPreferredSize(new java.awt.Dimension(16, 16));

        selectedFileLabel.setText(GeneratorDialogBundle.getSelectedFileText(Locale.getDefault()));

        chooseFileButton.setIcon(this.getIconSet().getIcon(IconSet.SAVE_AS, IconSet.SIZE_16X16, Locale.getDefault()));
        chooseFileButton.setBorderPainted(false);
        chooseFileButton.setContentAreaFilled(false);
        chooseFileButton.setDefaultCapable(false);
        chooseFileButton.setMaximumSize(new java.awt.Dimension(16, 16));
        chooseFileButton.setMinimumSize(new java.awt.Dimension(16, 16));
        chooseFileButton.setPreferredSize(new java.awt.Dimension(16, 16));
        chooseFileButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                chooseFileButtonActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout detailsPanelLayout = new org.jdesktop.layout.GroupLayout(detailsPanel);
        detailsPanel.setLayout(detailsPanelLayout);
        detailsPanelLayout.setHorizontalGroup(
            detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(detailsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(detailsPanelLayout.createSequentialGroup()
                        .add(headerBankLabel)
                        .add(289, 289, 289))
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, detailsPanelLayout.createSequentialGroup()
                        .add(detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, selectedFileLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 361, Short.MAX_VALUE)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, headerCustomerLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 361, Short.MAX_VALUE)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, numDescriptionsLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 361, Short.MAX_VALUE)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, numTransactionsLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 361, Short.MAX_VALUE)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, numFilesLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 361, Short.MAX_VALUE)
                            .add(org.jdesktop.layout.GroupLayout.LEADING, headerAccountLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 361, Short.MAX_VALUE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)))
                .add(detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(headerAccountTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 438, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, detailsPanelLayout.createSequentialGroup()
                        .add(numDescriptionsTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 206, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(randomDescriptionCheckBox, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 226, Short.MAX_VALUE))
                    .add(numTransactionsTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 438, Short.MAX_VALUE)
                    .add(numFilesTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 438, Short.MAX_VALUE)
                    .add(headerCustomerTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 438, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, detailsPanelLayout.createSequentialGroup()
                        .add(headerBankTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 416, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(searchHeaderBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, detailsPanelLayout.createSequentialGroup()
                        .add(selectedFileTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 416, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(chooseFileButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        detailsPanelLayout.setVerticalGroup(
            detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(detailsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(numFilesLabel)
                    .add(numFilesTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(numTransactionsLabel)
                    .add(numTransactionsTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(numDescriptionsLabel)
                    .add(randomDescriptionCheckBox)
                    .add(numDescriptionsTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(headerCustomerTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(headerCustomerLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(headerBankLabel)
                    .add(searchHeaderBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(headerBankTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(headerAccountTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(headerAccountLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(detailsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(chooseFileButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(selectedFileTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(selectedFileLabel))
                .addContainerGap(63, Short.MAX_VALUE))
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, detailsPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(layout.createSequentialGroup()
                        .add(generateButton)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(closeButton))
                    .add(org.jdesktop.layout.GroupLayout.LEADING, layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(fileTypePanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .add(executivePanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(textschluesselPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .add(targetPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(targetPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(executivePanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                    .add(fileTypePanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(textschluesselPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(detailsPanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(7, 7, 7)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(generateButton)
                    .add(closeButton))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void formPropertyChange(java.beans.PropertyChangeEvent evt)//GEN-FIRST:event_formPropertyChange
    {//GEN-HEADEREND:event_formPropertyChange
        if(evt.getPropertyName().equals("outputFile"))
        {
            this.selectedFileTextField.setText(evt.getNewValue() == null ?
                null : ((File) evt.getNewValue()).getAbsolutePath());

        }
    }//GEN-LAST:event_formPropertyChange

    private void chooseFileButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_chooseFileButtonActionPerformed
    {//GEN-HEADEREND:event_chooseFileButtonActionPerformed
        final JFileChooser chooser = this.getUserInterface().
            getOpenFileChooser();

        int status = chooser.showSaveDialog(this.getUserInterface().
            getParent());

        if(status == JFileChooser.APPROVE_OPTION)
        {
            this.setOutputFile(chooser.getSelectedFile());
        }
    }//GEN-LAST:event_chooseFileButtonActionPerformed

    private void fileTypeListValueChanged(javax.swing.event.ListSelectionEvent evt)//GEN-FIRST:event_fileTypeListValueChanged
    {//GEN-HEADEREND:event_fileTypeListValueChanged
        final Object[] selectedTypes = this.fileTypeList.getSelectedValues();
        final Object[] prevSelection = this.textschluesselList.
            getSelectedValues();

        this.getTextschluesselModel().clear();

        if(selectedTypes == null || selectedTypes.length == 0)
        {
            for(Textschluessel key : this.getSupportedTextschluessel())
            {
                this.getTextschluesselModel().addElement(key);
            }
        }
        else
        {
            boolean debitAllowed = false;
            boolean remittanceAllowed = false;

            for(int i = selectedTypes.length - 1; i >= 0; i--)
            {
                if(!debitAllowed)
                {
                    debitAllowed = ((LogicalFileType) selectedTypes[i]).
                        isDebitAllowed();

                }
                if(!remittanceAllowed)
                {
                    remittanceAllowed = ((LogicalFileType) selectedTypes[i]).
                        isRemittanceAllowed();

                }

                if(debitAllowed && remittanceAllowed)
                {
                    break;
                }
            }

            final ArrayList<Textschluessel> keys =
                new ArrayList<Textschluessel>(this.getSupportedTextschluessel().
                size());

            if(debitAllowed)
            {
                keys.addAll(Arrays.asList(this.getTextschluesselVerzeichnis().
                    search(true, false)));

                if(remittanceAllowed)
                {
                    keys.addAll(Arrays.asList(getTextschluesselVerzeichnis().
                        search(true, true)));

                }
            }

            if(remittanceAllowed)
            {
                keys.addAll(Arrays.asList(this.getTextschluesselVerzeichnis().
                    search(false, true)));

                if(debitAllowed)
                {
                    keys.addAll(Arrays.asList(getTextschluesselVerzeichnis().
                        search(true, true)));

                }
            }

            for(Textschluessel key : keys)
            {
                this.getTextschluesselModel().addElement(key);
            }
        }

        if(prevSelection != null)
        {
            final List<Integer> indicesList =
                new ArrayList<Integer>(prevSelection.length);

            for(int i = prevSelection.length - 1; i >= 0; i--)
            {
                final int index = this.getTextschluesselModel().
                    indexOf(prevSelection[i]);

                if(index >= 0)
                {
                    indicesList.add(index);
                }
            }

            int i = 0;
            final int[] indices = new int[indicesList.size()];
            for(Integer index : indicesList)
            {
                indices[i++] = index;
            }

            this.textschluesselList.setSelectedIndices(indices);
        }
    }//GEN-LAST:event_fileTypeListValueChanged

    private void generateButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_generateButtonActionPerformed
    {//GEN-HEADEREND:event_generateButtonActionPerformed
        if(this.selectedFileTextField.getText() != null)
        {
            this.outputFile = new File(this.selectedFileTextField.getText());
        }
        else
        {
            this.outputFile = null;
        }

        if(this.actionListener != null)
        {
            this.actionListener.actionPerformed(evt);
        }
    }//GEN-LAST:event_generateButtonActionPerformed

    private void randomDescriptionCheckBoxActionPerformed1(java.awt.event.ActionEvent evt)//GEN-FIRST:event_randomDescriptionCheckBoxActionPerformed1
    {//GEN-HEADEREND:event_randomDescriptionCheckBoxActionPerformed1
        numDescriptionsTextField.setValue(null);
    }//GEN-LAST:event_randomDescriptionCheckBoxActionPerformed1

    private void randomDescriptionCheckBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_randomDescriptionCheckBoxActionPerformed
    {//GEN-HEADEREND:event_randomDescriptionCheckBoxActionPerformed
        numDescriptionsTextField.setEnabled(!this.randomDescriptionCheckBox.isSelected());
    }//GEN-LAST:event_randomDescriptionCheckBoxActionPerformed

    private void closeButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_closeButtonActionPerformed
    {
        this.doClose(RET_CANCEL);
}//GEN-LAST:event_closeButtonActionPerformed

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt)//GEN-FIRST:event_closeDialog
    {
        if(!this.isLocked())
        {
            this.doClose(RET_CANCEL);
        }
    }//GEN-LAST:event_closeDialog

    private void doClose(final int retStatus)
    {
        this.returnStatus = retStatus;
        this.setVisible(false);
        this.dispose();
    }

    private List<? extends LogicalFileType> getSupportedFileTypes()
    {
        final List<LogicalFileType> types =
            new ArrayList<LogicalFileType>(50);

        types.addAll(Arrays.asList(LogicalFileType.search(
            false, false, false, false)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            false, false, true, false)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            false, false, false, true)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            false, false, true, true)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            true, false, false, false)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            true, false, true, false)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            true, false, false, true)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            true, false, true, true)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            false, true, false, false)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            false, true, true, false)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            false, true, false, true)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            false, true, true, true)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            true, true, false, false)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            true, true, true, false)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            true, true, false, true)));

        types.addAll(Arrays.asList(LogicalFileType.search(
            true, true, true, true)));

        return types;
    }

    private List<? extends Textschluessel> getSupportedTextschluessel()
    {
        final List<Textschluessel> keys = new ArrayList<Textschluessel>(50);

        keys.addAll(Arrays.asList(this.getTextschluesselVerzeichnis().
            search(false, false)));

        keys.addAll(Arrays.asList(this.getTextschluesselVerzeichnis().
            search(true, false)));

        keys.addAll(Arrays.asList(this.getTextschluesselVerzeichnis().
            search(false, true)));

        keys.addAll(Arrays.asList(this.getTextschluesselVerzeichnis().
            search(true, true)));

        return keys;
    }

    private void setForeground(final JComponent comp, final Color color)
    {
        if(!this.colors.containsKey(comp))
        {
            this.colors.put(comp, comp.getForeground());
        }
        comp.setForeground(color);
    }

    private void resetForeground(final JComponent comp)
    {
        if(this.colors.containsKey(comp))
        {
            comp.setForeground(this.colors.get(comp));
        }
    }

    private void handleIllegalGeneratorTask(final JComponent label,
        final JComponent comp, final Message[] msgs)
    {
        if(msgs.length > 0)
        {
            final StringBuffer tooltip = new StringBuffer(500).
                append("<html>");

            for(int i = 0; i < msgs.length; i++)
            {
                tooltip.append(msgs[i].getText(Locale.getDefault())).
                    append("<br>");

            }

            comp.setToolTipText(tooltip.append("</html>").toString());
            this.setForeground(label, Color.RED);
        }
        else
        {
            comp.setToolTipText(null);
            this.resetForeground(label);
        }
    }

    private DefaultListModel getTextschluesselModel()
    {
        return (DefaultListModel) this.textschluesselList.getModel();
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton chooseFileButton;
    private javax.swing.JButton closeButton;
    private javax.swing.JPanel detailsPanel;
    private javax.swing.JLabel executiveAccountLabel;
    private javax.swing.JTextField executiveAccountTextField;
    private javax.swing.JLabel executiveBankLabel;
    private javax.swing.JTextField executiveBankTextField;
    private javax.swing.JLabel executiveNameLabel;
    private javax.swing.JTextField executiveNameTextField;
    private javax.swing.JPanel executivePanel;
    private javax.swing.JList fileTypeList;
    private javax.swing.JPanel fileTypePanel;
    private javax.swing.JScrollPane fileTypeScrollPane;
    private javax.swing.JButton generateButton;
    private javax.swing.JLabel headerAccountLabel;
    private javax.swing.JTextField headerAccountTextField;
    private javax.swing.JLabel headerBankLabel;
    private javax.swing.JTextField headerBankTextField;
    private javax.swing.JLabel headerCustomerLabel;
    private javax.swing.JTextField headerCustomerTextField;
    private javax.swing.JLabel numDescriptionsLabel;
    private javax.swing.JFormattedTextField numDescriptionsTextField;
    private javax.swing.JLabel numFilesLabel;
    private javax.swing.JFormattedTextField numFilesTextField;
    private javax.swing.JLabel numTransactionsLabel;
    private javax.swing.JFormattedTextField numTransactionsTextField;
    private javax.swing.JCheckBox randomDescriptionCheckBox;
    private javax.swing.JButton searchExecutiveBankleitzahlButton;
    private javax.swing.JButton searchHeaderBankleitzahlButton;
    private javax.swing.JButton searchTargetBankleitzahlButton;
    private javax.swing.JLabel selectedFileLabel;
    private javax.swing.JTextField selectedFileTextField;
    private javax.swing.JLabel targetAccountLabel;
    private javax.swing.JTextField targetAccountTextField;
    private javax.swing.JLabel targetBankLabel;
    private javax.swing.JTextField targetBankTextField;
    private javax.swing.JLabel targetNameLabel;
    private javax.swing.JTextField targetNameTextField;
    private javax.swing.JPanel targetPanel;
    private javax.swing.JList textschluesselList;
    private javax.swing.JPanel textschluesselPanel;
    private javax.swing.JScrollPane textschluesselScrollPane;
    // End of variables declaration//GEN-END:variables

    private int returnStatus = RET_CANCEL;

    /**
     * Caches label colors.
     * @serial
     */
    private Map<JComponent, Color> colors = new HashMap(50);

    /**
     * Output file or {@code null}.
     */
    private transient File outputFile;

    /**
     * Flag indicating that the user interface is locked.
     * @serial
     */
    private boolean locked;

    /**
     * {@code ActionListener} called when the user requests the dialog's
     * operation.
     */
    private transient ActionListener actionListener;

    //---------------------------------------------------------GeneratorDialog--

}
