/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.banking.dtaus.swing;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Currency;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import org.jdtaus.banking.AlphaNumericText27;
import org.jdtaus.banking.Bankleitzahl;
import org.jdtaus.banking.BankleitzahlenVerzeichnis;
import org.jdtaus.banking.CurrencyDirectory;
import org.jdtaus.banking.Kontonummer;
import org.jdtaus.banking.Referenznummer10;
import org.jdtaus.banking.dtaus.Header;
import org.jdtaus.banking.dtaus.IllegalHeaderException;
import org.jdtaus.banking.dtaus.LogicalFileType;
import org.jdtaus.banking.swing.AlphaNumericText27TextField;
import org.jdtaus.banking.swing.BankleitzahlTextField;
import org.jdtaus.banking.swing.CurrencyRenderer;
import org.jdtaus.banking.swing.KontonummerTextField;
import org.jdtaus.banking.swing.Referenznummer10TextField;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ContextInitializer;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Properties;
import org.jdtaus.core.container.Property;
import org.jdtaus.core.text.Message;

/**
 * Dialog to display and edit a {@code Header}.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: HeaderDialog.java 3512 2007-05-16 05:57:21Z schulte2005 $
 */
public class HeaderDialog extends JDialog implements ContainerInitializer
{
    //--Implementation----------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(HeaderDialog.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /**
     * Initializes the properties of the instance.
     *
     * @param meta the property values to initialize the instance with.
     *
     * @throws NullPointerException if {@code meta} is {@code null}.
     */
    protected void initializeProperties(final Properties meta)
    {
        Property p;

        if(meta == null)
        {
            throw new NullPointerException("meta");
        }

    }

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    /** Initializes the instance centering the dialog on the screen. */
    public void initialize()
    {
        final Dimension screenSize = Toolkit.getDefaultToolkit().
            getScreenSize();

        this.initComponents();

        this.setLocation(screenSize.width / 2 - this.getWidth() / 2,
            screenSize.height / 2 - this.getHeight() / 2);

        this.updateCurrencies();
        this.updateFileTypes();
    }

    //----------------------------------------------------ContainerInitializer--
    //--Dependencies------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Configured <code>BankleitzahlenVerzeichnis</code> implementation. */
    private transient BankleitzahlenVerzeichnis _dependency3;

    /**
     * Gets the configured <code>BankleitzahlenVerzeichnis</code> implementation.
     *
     * @return the configured <code>BankleitzahlenVerzeichnis</code> implementation.
     */
    private BankleitzahlenVerzeichnis getBankleitzahlenVerzeichnis()
    {
        BankleitzahlenVerzeichnis ret = null;
        if(this._dependency3 != null)
        {
            ret = this._dependency3;
        }
        else
        {
            ret = (BankleitzahlenVerzeichnis) ContainerFactory.getContainer().
                getDependency(HeaderDialog.class,
                "BankleitzahlenVerzeichnis");

            if(ModelFactory.getModel().getModules().
                getImplementation(HeaderDialog.class.getName()).
                getDependencies().getDependency("BankleitzahlenVerzeichnis").
                isBound())
            {
                this._dependency3 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>UserInterface</code> implementation. */
    private transient EditorGUI _dependency2;

    /**
     * Gets the configured <code>UserInterface</code> implementation.
     *
     * @return the configured <code>UserInterface</code> implementation.
     */
    private EditorGUI getUserInterface()
    {
        EditorGUI ret = null;
        if(this._dependency2 != null)
        {
            ret = this._dependency2;
        }
        else
        {
            ret = (EditorGUI) ContainerFactory.getContainer().
                getDependency(HeaderDialog.class,
                "UserInterface");

            if(ModelFactory.getModel().getModules().
                getImplementation(HeaderDialog.class.getName()).
                getDependencies().getDependency("UserInterface").
                isBound())
            {
                this._dependency2 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>Application</code> implementation. */
    private transient Editor _dependency1;

    /**
     * Gets the configured <code>Application</code> implementation.
     *
     * @return the configured <code>Application</code> implementation.
     */
    private Editor getApplication()
    {
        Editor ret = null;
        if(this._dependency1 != null)
        {
            ret = this._dependency1;
        }
        else
        {
            ret = (Editor) ContainerFactory.getContainer().
                getDependency(HeaderDialog.class,
                "Application");

            if(ModelFactory.getModel().getModules().
                getImplementation(HeaderDialog.class.getName()).
                getDependencies().getDependency("Application").
                isBound())
            {
                this._dependency1 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>CurrencyDirectory</code> implementation. */
    private transient CurrencyDirectory _dependency0;

    /**
     * Gets the configured <code>CurrencyDirectory</code> implementation.
     *
     * @return the configured <code>CurrencyDirectory</code> implementation.
     */
    private CurrencyDirectory getCurrencyDirectory()
    {
        CurrencyDirectory ret = null;
        if(this._dependency0 != null)
        {
            ret = this._dependency0;
        }
        else
        {
            ret = (CurrencyDirectory) ContainerFactory.getContainer().
                getDependency(HeaderDialog.class,
                "CurrencyDirectory");

            if(ModelFactory.getModel().getModules().
                getImplementation(HeaderDialog.class.getName()).
                getDependencies().getDependency("CurrencyDirectory").
                isBound())
            {
                this._dependency0 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }

    //------------------------------------------------------------Dependencies--
    //--Properties--------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.


    //--------------------------------------------------------------Properties--
    //--HeaderDialog------------------------------------------------------------

    /** A return status code - returned if Cancel button has been pressed */
    public static final int RET_CANCEL = 0;

    /** A return status code - returned if OK button has been pressed */
    public static final int RET_OK = 1;

    /**
     * Creates new {@code HeaderDialog} instance.
     *
     * @param owner the frame owning the dialog.
     * @param modal {@code true} for a modal dialog; {@code false} else.
     */
    public HeaderDialog(final Frame owner, final boolean modal)
    {
        super(owner, modal);
        this.initializeProperties(HeaderDialog.META.getProperties());
        this.initialize();
    }

    /**
     * Gets the return status of the dialog.
     *
     * @return the return status of this dialog - one of {@code RET_OK} or
     * {@code RET_CANCEL}.
     */
    public int getReturnStatus()
    {
        return this.returnStatus;
    }

    /**
     * Gets the A record displayed by the dialog.
     *
     * @return the A record displayed by the dialog.
     *
     * @throws IllegalHeaderException if the dialog does not display a valid
     * A record.
     */
    public Header getHeader()
    {
        final Header h = new Header();
        final Preferences prefs = this.getApplication().getPreferences();
        final org.jdtaus.banking.dtaus.spi.IllegalHeaderException e =
            new org.jdtaus.banking.dtaus.spi.IllegalHeaderException();

        h.setCustomer(((AlphaNumericText27TextField) this.customerTextField).
            getAlphaNumericText27());

        h.setAccount(((KontonummerTextField) this.accountTextField).
            getKontonummer());

        h.setBank(((BankleitzahlTextField) this.bankTextField).
            getBankleitzahl());

        h.setReference(((Referenznummer10TextField) this.referenceTextField).
            getReferenznummer10());

        h.setCreateDate((Date) this.createDateTextField.getValue());
        h.setExecutionDate((Date) this.executionDateTextField.getValue());

        if(!(prefs.isBankcodeValidationDisabled() ||
            this.isBankcodeValid(h.getBank())))
        {
            e.addMessage(Header.PROP_BANK, new UnknownBankleitzahlMessage());
        }

        h.setType((LogicalFileType) this.typeComboBox.getSelectedItem());
        h.setCurrency((Currency) this.currencyComboBox.getSelectedItem());

        if(h.getType() != null && h.getType().isSendByBank())
        {
            h.setBankData(((BankleitzahlTextField) this.bankDataTextField).
                getBankleitzahl());

            if(!(prefs.isBankcodeValidationDisabled() ||
                this.isBankcodeValid(h.getBankData())))
            {
                e.addMessage(Header.PROP_BANKDATA,
                    new UnknownBankleitzahlMessage());

            }
        }

        if(e.getMessages().length > 0)
        {
            throw e;
        }

        return h;
    }

    /**
     * Sets the A record to be displayed by the dialog.
     *
     * @param header the A record to be displayed by the dialog or
     * {@code null} to reset the dialog display.
     */
    public void setHeader(final Header header)
    {
        this.date = header != null ? header.getCreateDate() : null;

        this.updateCurrencies();

        if(header == null)
        {
            ((AlphaNumericText27TextField) this.customerTextField).
                setValue(null);

            ((Referenznummer10TextField) this.referenceTextField).
                setValue(null);

            ((BankleitzahlTextField) this.bankDataTextField).setValue(null);
            ((KontonummerTextField) this.accountTextField).setValue(null);
            ((BankleitzahlTextField) this.bankTextField).setValue(null);
            this.currencyComboBox.setSelectedItem(null);
            this.typeComboBox.setSelectedItem(null);
            this.createDateTextField.setValue(null);
            this.executionDateTextField.setValue(null);
        }
        else
        {
            ((KontonummerTextField) this.accountTextField).
                setValue(header.getAccount());

            ((BankleitzahlTextField) this.bankTextField).
                setValue(header.getBank());

            ((AlphaNumericText27TextField) this.customerTextField).
                setValue(header.getCustomer());

            ((Referenznummer10TextField) this.referenceTextField).
                setValue(header.getReference());

            ((BankleitzahlTextField) this.bankDataTextField).
                setValue(header.getBankData());

            this.currencyComboBox.setSelectedItem(header.getCurrency());
            this.typeComboBox.setSelectedItem(header.getType());
            this.createDateTextField.setValue(header.getCreateDate());
            this.executionDateTextField.setValue(header.getExecutionDate());
        }
    }

    /**
     * Flag indicating if the dialog supports logical files holding debit
     * orders.
     *
     * @return {@code true} if the dialog supports logical files holding
     * debit orders; {@code false} if not.
     */
    public boolean isDebitSupported()
    {
        return this.debitSupported;
    }

    /**
     * Sets the flag indicating if the dialog supports logical files holding
     * debit orders.
     *
     * @param value {@code true} if the dialog should support logical files
     * holding debit orders; {@code false} if not.
     */
    public void setDebitSupported(final boolean value)
    {
        this.firePropertyChange("debitSupported", this.debitSupported, value);
        this.debitSupported = value;
        this.updateFileTypes();
    }

    /**
     * Flag indicating if the dialog supports logical files holding remittance
     * orders.
     *
     * @return {@code true} if the dialog supports logical files holding
     * remittance orders; {@code false} if not.
     */
    public boolean isRemittanceSupported()
    {
        return this.remittanceSupported;
    }

    /**
     * Sets the flag indicating if the dialog supports logical files holding
     * remittance orders.
     *
     * @param value {@code true} if the dialog should support logical files
     * holding remittance orders; {@code false} if not.
     */
    public void setRemittanceSupported(final boolean value)
    {
        this.firePropertyChange("remittanceSupported", this.remittanceSupported,
            value);

        this.remittanceSupported = value;
        this.updateFileTypes();
    }

    /**
     * Gets the {@code ActionListener} called when the user requests this
     * dialog's operation.
     *
     * @return the {@code ActionListener} called when the user requests this
     * dialog's operation or {@code null}.
     */
    public ActionListener getActionListener()
    {
        return this.actionListener;
    }

    /**
     * Sets the {@code ActionListener} to be called when the user requests this
     * dialog's operation.
     *
     * @param listener the {@code ActionListener} to be called when the user
     * requests this dialog's operation or {@code null}.
     */
    public void setActionListener(final ActionListener listener)
    {
        this.actionListener = listener;
    }

    /**
     * Updates the dialog according to a caught {@code IllegalHeaderException}.
     *
     * @param e the caught {@code IllegalHeaderException} to update the
     * dialog with or {@code null} to reset the dialog from any caught
     * {@code IllegalHeaderException}s.
     */
    public void onIllegalHeader(final IllegalHeaderException e)
    {
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                for(JLabel label : colors.keySet())
                {
                    resetForeground(label);
                }

                accountTextField.setToolTipText(null);
                bankDataTextField.setToolTipText(null);
                bankTextField.setToolTipText(null);
                createDateTextField.setToolTipText(null);
                currencyComboBox.setToolTipText(null);
                customerTextField.setToolTipText(null);
                executionDateTextField.setToolTipText(null);
                referenceTextField.setToolTipText(null);
                typeComboBox.setToolTipText(null);

                if(e != null)
                {
                    handleIllegalHeader(accountLabel, accountTextField,
                        e.getMessages(Header.PROP_ACCOUNT));

                    handleIllegalHeader(bankLabel, bankTextField,
                        e.getMessages(Header.PROP_BANK));

                    handleIllegalHeader(bankDataLabel, bankDataTextField,
                        e.getMessages(Header.PROP_BANKDATA));

                    handleIllegalHeader(createDateLabel, createDateTextField,
                        e.getMessages(Header.PROP_CREATEDATE));

                    handleIllegalHeader(currencyLabel, currencyComboBox,
                        e.getMessages(Header.PROP_CURRENCY));

                    handleIllegalHeader(customerLabel, customerTextField,
                        e.getMessages(Header.PROP_CUSTOMER));

                    handleIllegalHeader(executionDateLabel,
                        executionDateTextField,
                        e.getMessages(Header.PROP_EXECUTIONDATE));

                    handleIllegalHeader(referenceLabel, referenceTextField,
                        e.getMessages(Header.PROP_REFERENCE));

                    handleIllegalHeader(typeLabel, typeComboBox,
                        e.getMessages(Header.PROP_TYPE));

                    final Message[] msgs = e.getMessages();
                    if(msgs.length > 0)
                    {
                        final StringBuffer buf =
                            new StringBuffer(msgs.length * 150);

                        for(int i = 0; i < msgs.length; i++)
                        {
                            buf.append(msgs[i].getText(Locale.getDefault())).
                                append('\n');
                        }

                        JOptionPane.showMessageDialog(getUserInterface().
                            getParent(), buf.toString(), e.getMessage(),
                            JOptionPane.INFORMATION_MESSAGE);

                    }
                }
            }
        });
    }

    /**
     * Flag indicating if the dialog is locked for any interaction.
     *
     * @return {@code true} if the dialog is locked for any interaction;
     * {@code false} if the dialog is not locked.
     */
    public boolean isLocked()
    {
        return this.locked;
    }

    /**
     * Setter for property {@code locked}.
     *
     * @param locked {@code true} if the dialog should be locked for any
     * interaction; {@code false} to unlock the dialog.
     */
    public void setLocked(final boolean locked)
    {
        this.setCursor(Cursor.getPredefinedCursor(locked ?
            Cursor.WAIT_CURSOR : Cursor.DEFAULT_CURSOR));

        this.accountTextField.setEnabled(!locked);
        this.bankDataTextField.setEnabled(!locked);
        this.bankTextField.setEnabled(!locked);
        this.cancelButton.setEnabled(!locked);
        this.createDateTextField.setEnabled(!locked);
        this.currencyComboBox.setEnabled(!locked);
        this.customerTextField.setEnabled(!locked);
        this.executionDateTextField.setEnabled(!locked);
        this.okButton.setEnabled(!locked);
        this.referenceTextField.setEnabled(!locked);

        this.firePropertyChange("locked", this.locked, locked);
        this.locked = locked;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        okButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        headerPanel = new javax.swing.JPanel();
        typeLabel = new javax.swing.JLabel();
        typeComboBox = new javax.swing.JComboBox();
        customerLabel = new javax.swing.JLabel();
        customerTextField = new AlphaNumericText27TextField();
        accountLabel = new javax.swing.JLabel();
        accountTextField = new KontonummerTextField();
        bankLabel = new javax.swing.JLabel();
        bankTextField = new BankleitzahlTextField();
        currencyLabel = new javax.swing.JLabel();
        currencyComboBox = new javax.swing.JComboBox();
        referenceLabel = new javax.swing.JLabel();
        referenceTextField = new Referenznummer10TextField();
        createDateLabel = new javax.swing.JLabel();
        createDateTextField = new javax.swing.JFormattedTextField();
        executionDateLabel = new javax.swing.JLabel();
        executionDateTextField = new javax.swing.JFormattedTextField();
        searchBankleitzahlButton = new javax.swing.JButton();
        bankDataLabel = new javax.swing.JLabel();
        bankDataTextField = new BankleitzahlTextField();
        searchBankDataButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle(HeaderDialogBundle.getTitleText(Locale.getDefault()));
        setName("Header Dialog");
        addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(java.awt.event.WindowEvent evt)
            {
                closeDialog(evt);
            }
        });

        okButton.setText("OK");
        okButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                okButtonActionPerformed(evt);
            }
        });

        cancelButton.setText("Cancel");
        cancelButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                cancelButtonActionPerformed(evt);
            }
        });

        headerPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(HeaderDialogBundle.getHeaderPanelTitleText(Locale.getDefault())));
        typeLabel.setText(HeaderDialogBundle.getTypeLabelText(Locale.getDefault()));

        typeComboBox.setRenderer(new LogicalFileTypeRenderer());
        typeComboBox.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                typeComboBoxActionPerformed(evt);
            }
        });

        customerLabel.setText(HeaderDialogBundle.getCustomerLabelText(Locale.getDefault()));

        customerTextField.setColumns(AlphaNumericText27.MAX_LENGTH);

        accountLabel.setText(HeaderDialogBundle.getAccountLabelText(Locale.getDefault()));

        accountTextField.setColumns(Kontonummer.MAX_CHARACTERS);

        bankLabel.setText(HeaderDialogBundle.getBankLabelText(Locale.getDefault()));

        bankTextField.setColumns(Bankleitzahl.MAX_CHARACTERS);

        currencyLabel.setText(HeaderDialogBundle.getCurrencyLabelText(Locale.getDefault()));

        currencyComboBox.setRenderer(new CurrencyRenderer());

        referenceLabel.setText(HeaderDialogBundle.getReferenceLabelText(Locale.getDefault()));

        referenceTextField.setColumns(Referenznummer10.MAX_CHARACTERS);

        createDateLabel.setText(HeaderDialogBundle.getCreateDateLabelText(Locale.getDefault()));

        createDateTextField.setValue(new Date());
        createDateTextField.addPropertyChangeListener(new java.beans.PropertyChangeListener()
        {
            public void propertyChange(java.beans.PropertyChangeEvent evt)
            {
                createDateTextFieldPropertyChange(evt);
            }
        });

        executionDateLabel.setText(HeaderDialogBundle.getExecutionDateLabelText(Locale.getDefault()));

        executionDateTextField.setValue(new Date());

        searchBankleitzahlButton.setAction(new SearchBankleitzahlAction(this.bankTextField, true));
        searchBankleitzahlButton.setBorderPainted(false);
        searchBankleitzahlButton.setContentAreaFilled(false);
        searchBankleitzahlButton.setDefaultCapable(false);
        searchBankleitzahlButton.setMaximumSize(new java.awt.Dimension(16, 16));
        searchBankleitzahlButton.setMinimumSize(new java.awt.Dimension(16, 16));
        searchBankleitzahlButton.setPreferredSize(new java.awt.Dimension(16, 16));

        bankDataLabel.setText(HeaderDialogBundle.getBankDataLabelText(Locale.getDefault()));

        bankDataTextField.setEnabled(false);

        searchBankDataButton.setAction(new SearchBankleitzahlAction(this.bankDataTextField, true));
        searchBankDataButton.setBorderPainted(false);
        searchBankDataButton.setContentAreaFilled(false);
        searchBankDataButton.setDefaultCapable(false);
        searchBankDataButton.setMaximumSize(new java.awt.Dimension(16, 16));
        searchBankDataButton.setMinimumSize(new java.awt.Dimension(16, 16));
        searchBankDataButton.setPreferredSize(new java.awt.Dimension(16, 16));

        org.jdesktop.layout.GroupLayout headerPanelLayout = new org.jdesktop.layout.GroupLayout(headerPanel);
        headerPanel.setLayout(headerPanelLayout);
        headerPanelLayout.setHorizontalGroup(
            headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(headerPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, bankDataLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, executionDateLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, createDateLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, referenceLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, currencyLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, bankLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, accountLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, customerLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, typeLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 135, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(createDateTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 319, Short.MAX_VALUE)
                    .add(referenceTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 319, Short.MAX_VALUE)
                    .add(currencyComboBox, 0, 319, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, headerPanelLayout.createSequentialGroup()
                        .add(bankTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 297, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(searchBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(accountTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 319, Short.MAX_VALUE)
                    .add(customerTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 319, Short.MAX_VALUE)
                    .add(typeComboBox, 0, 319, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, executionDateTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 319, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, headerPanelLayout.createSequentialGroup()
                        .add(bankDataTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 297, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(searchBankDataButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        headerPanelLayout.setVerticalGroup(
            headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(headerPanelLayout.createSequentialGroup()
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(typeComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(typeLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(customerTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(customerLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(accountTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(accountLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(bankLabel)
                    .add(bankTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(searchBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(currencyComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(currencyLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(referenceTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(referenceLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(createDateTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(createDateLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(executionDateLabel)
                    .add(executionDateTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(headerPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(bankDataLabel)
                    .add(searchBankDataButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(bankDataTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(38, 38, 38))
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap(356, Short.MAX_VALUE)
                .add(okButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 67, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(cancelButton)
                .addContainerGap())
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(headerPanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        layout.linkSize(new java.awt.Component[] {cancelButton, okButton}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(headerPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 292, Short.MAX_VALUE)
                .add(16, 16, 16)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(cancelButton)
                    .add(okButton))
                .addContainerGap())
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void createDateTextFieldPropertyChange(java.beans.PropertyChangeEvent evt)//GEN-FIRST:event_createDateTextFieldPropertyChange
    {//GEN-HEADEREND:event_createDateTextFieldPropertyChange
        if(evt.getPropertyName().equals("value"))
        {
            this.date = (Date) this.createDateTextField.getValue();
            this.updateCurrencies();
        }
    }//GEN-LAST:event_createDateTextFieldPropertyChange

    private void typeComboBoxActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_typeComboBoxActionPerformed
    {//GEN-HEADEREND:event_typeComboBoxActionPerformed
        final LogicalFileType type =
            (LogicalFileType) this.typeComboBox.getSelectedItem();

        final boolean enabled = type != null && type.isSendByBank();
        bankDataTextField.setEnabled(enabled);
        searchBankDataButton.setEnabled(enabled);
    }//GEN-LAST:event_typeComboBoxActionPerformed

    private void okButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_okButtonActionPerformed
    {
        if(this.getActionListener() != null)
        {
            this.getActionListener().actionPerformed(evt);
        }
    }//GEN-LAST:event_okButtonActionPerformed

    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cancelButtonActionPerformed
    {
        this.doClose(RET_CANCEL);
    }//GEN-LAST:event_cancelButtonActionPerformed

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt)//GEN-FIRST:event_closeDialog
    {
        if(!this.isLocked())
        {
            this.doClose(RET_CANCEL);
        }
    }//GEN-LAST:event_closeDialog

    private void doClose(int retStatus)
    {
        this.returnStatus = retStatus;
        this.setVisible(false);
        this.dispose();
    }

    private void setForeground(final JLabel label, final Color color)
    {
        if(!this.colors.containsKey(label))
        {
            this.colors.put(label, label.getForeground());
        }
        label.setForeground(color);
    }

    private void resetForeground(final JLabel label)
    {
        if(this.colors.containsKey(label))
        {
            label.setForeground(this.colors.get(label));
        }
    }

    private void handleIllegalHeader(final JLabel label,
        final JComponent comp, final Message[] msgs)
    {
        if(msgs.length > 0)
        {
            final StringBuffer tooltip = new StringBuffer(500).
                append("<html>");

            for(int i = 0; i < msgs.length; i++)
            {
                tooltip.append(msgs[i].getText(Locale.getDefault())).
                    append("<br>");

            }

            comp.setToolTipText(tooltip.append("</html>").toString());
            this.setForeground(label, Color.RED);
        }
        else
        {
            comp.setToolTipText(null);
            this.resetForeground(label);
        }
    }

    private void updateCurrencies()
    {
        final Currency[] currencies = getCurrencyDirectory().
            getDtausCurrencies(this.date != null ? this.date : new Date());

        Arrays.sort(currencies, new Comparator()
        {
            public int compare(final Object o1, final Object o2)
            {
                assert o1 instanceof Currency;
                assert o2 instanceof Currency;

                return ((Currency) o1).getCurrencyCode().
                    compareTo(((Currency) o2).getCurrencyCode());

            }
        });

        final Object prevSelection = currencyComboBox.getSelectedItem();
        currencyComboBox.removeAllItems();
        for(int i = 0; i < currencies.length; i++)
        {
            currencyComboBox.addItem(currencies[i]);
        }

        currencyComboBox.setSelectedItem(prevSelection);
    }

    private void updateFileTypes()
    {
        final List<LogicalFileType> types =
            new ArrayList<LogicalFileType>(50);

        if(isDebitSupported())
        {
            types.addAll(Arrays.asList(LogicalFileType.
                search(true, false, true, false)));

            types.addAll(Arrays.asList(LogicalFileType.
                search(true, false, false, true)));

            types.addAll(Arrays.asList(LogicalFileType.
                search(true, false, true, true)));

            if(isRemittanceSupported())
            {
                types.addAll(Arrays.asList(LogicalFileType.
                    search(true, true, true, false)));

                types.addAll(Arrays.asList(LogicalFileType.
                    search(true, true, false, true)));

                types.addAll(Arrays.asList(LogicalFileType.
                    search(true, true, true, true)));

            }
        }
        if(isRemittanceSupported())
        {
            types.addAll(Arrays.asList(LogicalFileType.
                search(false, true, true, false)));

            types.addAll(Arrays.asList(LogicalFileType.
                search(false, true, false, true)));

            types.addAll(Arrays.asList(LogicalFileType.
                search(false, true, true, true)));

            if(isDebitSupported())
            {
                types.addAll(Arrays.asList(LogicalFileType.
                    search(true, true, true, false)));

                types.addAll(Arrays.asList(LogicalFileType.
                    search(true, true, false, true)));

                types.addAll(Arrays.asList(LogicalFileType.
                    search(true, true, true, true)));

            }
        }

        final Object prevSelection = typeComboBox.getSelectedItem();
        typeComboBox.removeAllItems();
        for(final LogicalFileType type : types)
        {
            typeComboBox.addItem(type);
        }

        typeComboBox.setSelectedItem(prevSelection);
    }

    private boolean isBankcodeValid(final Bankleitzahl bankcode)
    {
        boolean valid = bankcode != null;

        if(valid)
        {
            valid = this.getBankleitzahlenVerzeichnis().
                getHeadOffice(bankcode) != null;

        }

        return valid;
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel accountLabel;
    private javax.swing.JTextField accountTextField;
    private javax.swing.JLabel bankDataLabel;
    private javax.swing.JTextField bankDataTextField;
    private javax.swing.JLabel bankLabel;
    private javax.swing.JTextField bankTextField;
    private javax.swing.JButton cancelButton;
    private javax.swing.JLabel createDateLabel;
    private javax.swing.JFormattedTextField createDateTextField;
    private javax.swing.JComboBox currencyComboBox;
    private javax.swing.JLabel currencyLabel;
    private javax.swing.JLabel customerLabel;
    private javax.swing.JTextField customerTextField;
    private javax.swing.JLabel executionDateLabel;
    private javax.swing.JFormattedTextField executionDateTextField;
    private javax.swing.JPanel headerPanel;
    private javax.swing.JButton okButton;
    private javax.swing.JLabel referenceLabel;
    private javax.swing.JTextField referenceTextField;
    private javax.swing.JButton searchBankDataButton;
    private javax.swing.JButton searchBankleitzahlButton;
    private javax.swing.JComboBox typeComboBox;
    private javax.swing.JLabel typeLabel;
    // End of variables declaration//GEN-END:variables

    private int returnStatus = RET_CANCEL;

    /**
     * Caches label colors.
     * @serial
     */
    private Map<JLabel, Color> colors = new HashMap(50);

    /**
     * Flag indicating if the dialog supports logical files which may hold
     * debit orders.
     * @serial
     */
    private boolean debitSupported;

    /**
     * Flag indicating if the dialog supports logical files which may hold
     * remittance orders.
     * @serial
     */
    private boolean remittanceSupported;


    /**
     * Date to be used for calculating the available currencies.
     * @serial
     */
    private Date date;

    /**
     * Flag indicating that the user interface is locked.
     * @serial
     */
    private boolean locked;

    /**
     * {@code ActionListener} called when the user requests the dialog's
     * operation.
     */
    private transient ActionListener actionListener;

}
