/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.banking.dtaus.swing;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.event.ActionListener;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Currency;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.text.JTextComponent;
import org.jdtaus.banking.AlphaNumericText27;
import org.jdtaus.banking.Bankleitzahl;
import org.jdtaus.banking.BankleitzahlenVerzeichnis;
import org.jdtaus.banking.CurrencyDirectory;
import org.jdtaus.banking.Kontonummer;
import org.jdtaus.banking.Referenznummer11;
import org.jdtaus.banking.Textschluessel;
import org.jdtaus.banking.TextschluesselVerzeichnis;
import org.jdtaus.banking.dtaus.IllegalTransactionException;
import org.jdtaus.banking.dtaus.Transaction;
import org.jdtaus.banking.swing.AlphaNumericText27TextField;
import org.jdtaus.banking.swing.BankleitzahlTextField;
import org.jdtaus.banking.swing.CurrencyRenderer;
import org.jdtaus.banking.swing.KontonummerTextField;
import org.jdtaus.banking.swing.Referenznummer11TextField;
import org.jdtaus.banking.swing.TextschluesselRenderer;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ContextInitializer;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Properties;
import org.jdtaus.core.container.Property;
import org.jdtaus.core.text.Message;

/**
 * Dialog to display and edit a {@code Transaction}.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: TransactionDialog.java 3512 2007-05-16 05:57:21Z schulte2005 $
 */
public class TransactionDialog extends JDialog implements ContainerInitializer
{
    //--Implementation----------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(TransactionDialog.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /**
     * Initializes the properties of the instance.
     *
     * @param meta the property values to initialize the instance with.
     *
     * @throws NullPointerException if {@code meta} is {@code null}.
     */
    protected void initializeProperties(final Properties meta)
    {
        Property p;

        if(meta == null)
        {
            throw new NullPointerException("meta");
        }

    }

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    /** Initializes the instance centering the dialog on the screen. */
    public void initialize()
    {
        final Dimension screenSize = Toolkit.getDefaultToolkit().
            getScreenSize();

        this.initComponents();

        this.setLocation(screenSize.width / 2 - this.getWidth() / 2,
            screenSize.height / 2 - this.getHeight() / 2);

        this.updateModel();
        this.updateCurrencies();
    }

    //----------------------------------------------------ContainerInitializer--
    //--Dependencies------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Configured <code>BankleitzahlenVerzeichnis</code> implementation. */
    private transient BankleitzahlenVerzeichnis _dependency4;

    /**
     * Gets the configured <code>BankleitzahlenVerzeichnis</code> implementation.
     *
     * @return the configured <code>BankleitzahlenVerzeichnis</code> implementation.
     */
    private BankleitzahlenVerzeichnis getBankleitzahlenVerzeichnis()
    {
        BankleitzahlenVerzeichnis ret = null;
        if(this._dependency4 != null)
        {
            ret = this._dependency4;
        }
        else
        {
            ret = (BankleitzahlenVerzeichnis) ContainerFactory.getContainer().
                getDependency(TransactionDialog.class,
                "BankleitzahlenVerzeichnis");

            if(ModelFactory.getModel().getModules().
                getImplementation(TransactionDialog.class.getName()).
                getDependencies().getDependency("BankleitzahlenVerzeichnis").
                isBound())
            {
                this._dependency4 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>UserInterface</code> implementation. */
    private transient EditorGUI _dependency3;

    /**
     * Gets the configured <code>UserInterface</code> implementation.
     *
     * @return the configured <code>UserInterface</code> implementation.
     */
    private EditorGUI getUserInterface()
    {
        EditorGUI ret = null;
        if(this._dependency3 != null)
        {
            ret = this._dependency3;
        }
        else
        {
            ret = (EditorGUI) ContainerFactory.getContainer().
                getDependency(TransactionDialog.class,
                "UserInterface");

            if(ModelFactory.getModel().getModules().
                getImplementation(TransactionDialog.class.getName()).
                getDependencies().getDependency("UserInterface").
                isBound())
            {
                this._dependency3 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>Application</code> implementation. */
    private transient Editor _dependency2;

    /**
     * Gets the configured <code>Application</code> implementation.
     *
     * @return the configured <code>Application</code> implementation.
     */
    private Editor getApplication()
    {
        Editor ret = null;
        if(this._dependency2 != null)
        {
            ret = this._dependency2;
        }
        else
        {
            ret = (Editor) ContainerFactory.getContainer().
                getDependency(TransactionDialog.class,
                "Application");

            if(ModelFactory.getModel().getModules().
                getImplementation(TransactionDialog.class.getName()).
                getDependencies().getDependency("Application").
                isBound())
            {
                this._dependency2 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>CurrencyDirectory</code> implementation. */
    private transient CurrencyDirectory _dependency1;

    /**
     * Gets the configured <code>CurrencyDirectory</code> implementation.
     *
     * @return the configured <code>CurrencyDirectory</code> implementation.
     */
    private CurrencyDirectory getCurrencyDirectory()
    {
        CurrencyDirectory ret = null;
        if(this._dependency1 != null)
        {
            ret = this._dependency1;
        }
        else
        {
            ret = (CurrencyDirectory) ContainerFactory.getContainer().
                getDependency(TransactionDialog.class,
                "CurrencyDirectory");

            if(ModelFactory.getModel().getModules().
                getImplementation(TransactionDialog.class.getName()).
                getDependencies().getDependency("CurrencyDirectory").
                isBound())
            {
                this._dependency1 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }
    /** Configured <code>TextschluesselVerzeichnis</code> implementation. */
    private transient TextschluesselVerzeichnis _dependency0;

    /**
     * Gets the configured <code>TextschluesselVerzeichnis</code> implementation.
     *
     * @return the configured <code>TextschluesselVerzeichnis</code> implementation.
     */
    private TextschluesselVerzeichnis getTextschluesselVerzeichnis()
    {
        TextschluesselVerzeichnis ret = null;
        if(this._dependency0 != null)
        {
            ret = this._dependency0;
        }
        else
        {
            ret = (TextschluesselVerzeichnis) ContainerFactory.getContainer().
                getDependency(TransactionDialog.class,
                "TextschluesselVerzeichnis");

            if(ModelFactory.getModel().getModules().
                getImplementation(TransactionDialog.class.getName()).
                getDependencies().getDependency("TextschluesselVerzeichnis").
                isBound())
            {
                this._dependency0 = ret;
            }
        }

        if(ret instanceof ContextInitializer && !((ContextInitializer) ret).
            isInitialized(ContextFactory.getContext()))
        {
            ((ContextInitializer) ret).initialize(ContextFactory.getContext());
        }

        return ret;
    }

    //------------------------------------------------------------Dependencies--
    //--Properties--------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.


    //--------------------------------------------------------------Properties--
    //--TransactionDialog-------------------------------------------------------

    /** A return status code - returned if Cancel button has been pressed */
    public static final int RET_CANCEL = 0;

    /** A return status code - returned if OK button has been pressed */
    public static final int RET_OK = 1;

    /** Number of description text components of the dialog. */
    private static final int NUM_DESCRIPTIONS = 14;

    /**
     * Creates new {@code TransactionDialog} instance.
     *
     * @param owner the frame owning the dialog.
     * @param modal {@code true} for a modal dialog; {@code false} else.
     */
    public TransactionDialog(final Frame owner, final boolean modal)
    {
        super(owner, modal);
        this.initializeProperties(TransactionDialog.META.getProperties());
        this.initialize();
    }

    /**
     * Gets the return status of the dialog.
     *
     * @return the return status of this dialog - one of {@code RET_OK} or
     * {@code RET_CANCEL}.
     */
    public int getReturnStatus()
    {
        return this.returnStatus;
    }

    /**
     * Gets the transaction displayed by the dialog.
     *
     * @return the transaction displayed by the dialog.
     *
     * @throws IllegalTransactionException if the dialog does not display a
     * valid C record.
     */
    public Transaction getTransaction()
    {
        final Transaction t = new Transaction();
        final NumberFormat amountFmt = NumberFormat.getNumberInstance();
        final Currency cur = (Currency) this.currencyComboBox.getSelectedItem();
        final Preferences prefs = this.getApplication().getPreferences();
        final List descriptions = new ArrayList(14);
        final org.jdtaus.banking.dtaus.spi.IllegalTransactionException e =
            new org.jdtaus.banking.dtaus.spi.IllegalTransactionException();

        t.setCurrency(cur);

        t.setExecutiveName(
            ((AlphaNumericText27TextField) this.executiveNameTextField).
            getAlphaNumericText27());

        t.setExecutiveExt(
            ((AlphaNumericText27TextField) this.executiveExtTextField).
            getAlphaNumericText27());

        t.setExecutiveAccount(
            ((KontonummerTextField) this.executiveAccountTextField).
            getKontonummer());

        t.setExecutiveBank(
            ((BankleitzahlTextField) this.executiveBankTextField).
            getBankleitzahl());

        t.setTargetName(
            ((AlphaNumericText27TextField) this.targetNameTextField).
            getAlphaNumericText27());

        t.setTargetExt(
            ((AlphaNumericText27TextField) this.targetExtTextField).
            getAlphaNumericText27());

        t.setTargetAccount(
            ((KontonummerTextField) this.targetAccountTextField).
            getKontonummer());

        t.setTargetBank(
            ((BankleitzahlTextField) this.targetBankTextField).
            getBankleitzahl());

        t.setReference(
            ((Referenznummer11TextField) this.referenceTextField).
            getReferenznummer11());

        AlphaNumericText27 txt =
            ((AlphaNumericText27TextField) this.descriptionTextField1).
            getAlphaNumericText27();

        if(txt != null && !txt.isEmpty())
        {
            descriptions.add(txt);
        }

        txt = ((AlphaNumericText27TextField) this.descriptionTextField2).
            getAlphaNumericText27();

        if(txt != null && !txt.isEmpty())
        {
            descriptions.add(txt);
        }

        txt = ((AlphaNumericText27TextField) this.descriptionTextField3).
            getAlphaNumericText27();

        if(txt != null && !txt.isEmpty())
        {
            descriptions.add(txt);
        }

        txt = ((AlphaNumericText27TextField) this.descriptionTextField4).
            getAlphaNumericText27();

        if(txt != null && !txt.isEmpty())
        {
            descriptions.add(txt);
        }

        txt = ((AlphaNumericText27TextField) this.descriptionTextField5).
            getAlphaNumericText27();

        if(txt != null && !txt.isEmpty())
        {
            descriptions.add(txt);
        }

        txt = ((AlphaNumericText27TextField) this.descriptionTextField6).
            getAlphaNumericText27();

        if(txt != null && !txt.isEmpty())
        {
            descriptions.add(txt);
        }

        txt = ((AlphaNumericText27TextField) this.descriptionTextField7).
            getAlphaNumericText27();

        if(txt != null && !txt.isEmpty())
        {
            descriptions.add(txt);
        }

        txt = ((AlphaNumericText27TextField) this.descriptionTextField8).
            getAlphaNumericText27();

        if(txt != null && !txt.isEmpty())
        {
            descriptions.add(txt);
        }

        txt = ((AlphaNumericText27TextField) this.descriptionTextField9).
            getAlphaNumericText27();

        if(txt != null && !txt.isEmpty())
        {
            descriptions.add(txt);
        }

        txt = ((AlphaNumericText27TextField) this.descriptionTextField10).
            getAlphaNumericText27();

        if(txt != null && !txt.isEmpty())
        {
            descriptions.add(txt);
        }

        txt = ((AlphaNumericText27TextField) this.descriptionTextField11).
            getAlphaNumericText27();

        if(txt != null && !txt.isEmpty())
        {
            descriptions.add(txt);
        }

        txt = ((AlphaNumericText27TextField) this.descriptionTextField12).
            getAlphaNumericText27();

        if(txt != null && !txt.isEmpty())
        {
            descriptions.add(txt);
        }

        txt = ((AlphaNumericText27TextField) this.descriptionTextField13).
            getAlphaNumericText27();

        if(txt != null && !txt.isEmpty())
        {
            descriptions.add(txt);
        }

        txt = ((AlphaNumericText27TextField) this.descriptionTextField14).
            getAlphaNumericText27();

        if(txt != null && !txt.isEmpty())
        {
            descriptions.add(txt);
        }

        t.setType((Textschluessel) this.textschluesselComboBox.
            getSelectedItem());

        if(!(prefs.isBankcodeValidationDisabled() ||
            isBankcodeValid(t.getExecutiveBank())))
        {
            e.addMessage(Transaction.PROP_EXECUTIVEBANK,
                new UnknownBankleitzahlMessage());

        }

        if(!(prefs.isBankcodeValidationDisabled() ||
            isBankcodeValid(t.getTargetBank())))
        {
            e.addMessage(Transaction.PROP_TARGETBANK,
                new UnknownBankleitzahlMessage());
        }

        if(cur != null)
        {
            amountFmt.setMaximumFractionDigits(
                cur.getDefaultFractionDigits());

            amountFmt.setMinimumFractionDigits(
                cur.getDefaultFractionDigits());

            try
            {
                t.setAmount(BigDecimal.valueOf(amountFmt.parse(
                    this.amountTextField.getText()).doubleValue()).
                    multiply(new BigDecimal(Math.pow(10,
                    cur.getDefaultFractionDigits()))).toBigInteger());

            }
            catch(ParseException ex)
            {
                t.setAmount(null);
            }
        }

        t.setDescriptions((AlphaNumericText27[]) descriptions.
            toArray(new AlphaNumericText27[descriptions.size()]));

        if(e.getMessages().length > 0)
        {
            throw e;
        }

        return t;
    }

    /**
     * Sets the transaction to be displayed by the dialog.
     *
     * @param value the transaction to be displayed by the dialog or
     * {@code null} to reset the dialog display.
     */
    public void setTransaction(final Transaction value)
    {
        AlphaNumericText27 txt;
        JTextComponent txtComp;

        if(value == null)
        {
            ((KontonummerTextField) this.executiveAccountTextField).
                setValue(null);

            ((BankleitzahlTextField) this.executiveBankTextField).
                setValue(null);

            ((AlphaNumericText27TextField) this.executiveExtTextField).
                setValue(null);

            ((AlphaNumericText27TextField) this.executiveNameTextField).
                setValue(null);

            ((KontonummerTextField) this.targetAccountTextField).setValue(null);
            ((BankleitzahlTextField) this.targetBankTextField).setValue(null);
            ((AlphaNumericText27TextField) this.targetExtTextField).
                setValue(null);

            ((AlphaNumericText27TextField) this.targetNameTextField).
                setValue(null);

            this.amountTextField.setText(null);
            ((Referenznummer11TextField) this.referenceTextField).
                setValue(null);

            this.textschluesselComboBox.setSelectedItem(null);
            this.currencyComboBox.setSelectedItem(null);
            ((AlphaNumericText27TextField) this.descriptionTextField1).
                setValue(null);


            ((AlphaNumericText27TextField) this.descriptionTextField1).
                setValue(null);

            ((AlphaNumericText27TextField) this.descriptionTextField2).
                setValue(null);

            ((AlphaNumericText27TextField) this.descriptionTextField3).
                setValue(null);

            ((AlphaNumericText27TextField) this.descriptionTextField4).
                setValue(null);

            ((AlphaNumericText27TextField) this.descriptionTextField5).
                setValue(null);

            ((AlphaNumericText27TextField) this.descriptionTextField6).
                setValue(null);

            ((AlphaNumericText27TextField) this.descriptionTextField7).
                setValue(null);

            ((AlphaNumericText27TextField) this.descriptionTextField8).
                setValue(null);

            ((AlphaNumericText27TextField) this.descriptionTextField9).
                setValue(null);

            ((AlphaNumericText27TextField) this.descriptionTextField10).
                setValue(null);

            ((AlphaNumericText27TextField) this.descriptionTextField11).
                setValue(null);

            ((AlphaNumericText27TextField) this.descriptionTextField12).
                setValue(null);

            ((AlphaNumericText27TextField) this.descriptionTextField13).
                setValue(null);

            ((AlphaNumericText27TextField) this.descriptionTextField14).
                setValue(null);

        }
        else
        {
            final Currency cur = value.getCurrency();
            if(cur != null && value.getAmount() != null)
            {
                final NumberFormat amountFmt = NumberFormat.getNumberInstance();

                amountFmt.setMaximumFractionDigits(
                    cur.getDefaultFractionDigits());

                amountFmt.setMinimumFractionDigits(
                    cur.getDefaultFractionDigits());

                final BigDecimal amount = new BigDecimal(value.getAmount()).
                    divide(new BigDecimal(Math.pow(10,
                    cur.getDefaultFractionDigits())).
                    setScale(cur.getDefaultFractionDigits(),
                    RoundingMode.HALF_UP));

                this.amountTextField.setText(amountFmt.format(amount));
            }
            else
            {
                this.amountTextField.setText(null);
            }

            this.currencyComboBox.setSelectedItem(value.getCurrency());

            ((KontonummerTextField) this.executiveAccountTextField).
                setValue(value.getExecutiveAccount());

            ((BankleitzahlTextField) this.executiveBankTextField).
                setValue(value.getExecutiveBank());

            ((AlphaNumericText27TextField) this.executiveNameTextField).
                setValue(value.getExecutiveName());


            ((AlphaNumericText27TextField) this.executiveExtTextField).
                setValue(value.getExecutiveExt());

            ((KontonummerTextField) this.targetAccountTextField).
                setValue(value.getTargetAccount());

            ((BankleitzahlTextField) this.targetBankTextField).
                setValue(value.getTargetBank());

            ((AlphaNumericText27TextField) this.targetNameTextField).
                setValue(value.getTargetName());

            ((AlphaNumericText27TextField) this.targetExtTextField).
                setValue(value.getTargetExt());

            ((Referenznummer11TextField) this.referenceTextField).
                setValue(value.getReference());

            this.textschluesselComboBox.setSelectedItem(value.getType());

            for(int i = TransactionDialog.NUM_DESCRIPTIONS; i > 0; i--)
            {
                txtComp = this.getDescriptionComponent(i);
                if(i - 1 < value.getDescriptions().length)
                {
                    txt = value.getDescriptions()[i - 1];
                    ((AlphaNumericText27TextField) txtComp).setValue(txt);
                }
                else
                {
                    ((AlphaNumericText27TextField) txtComp).setValue(null);
                }
            }
        }
    }

    /**
     * Flag indicating if the dialog supports debit {@code Textschluessel}.
     *
     * @return {@code true} if the dialog supports debit {@code Textschluessel};
     * {@code false} if not.
     */
    public boolean isDebit()
    {
        return this.debit;
    }

    /**
     * Setter for property {@code debit}.
     *
     * @param debit {@code true} if the dialog supports debit
     * {@code Textschluessel}; {@code false} if not.
     */
    public void setDebit(final boolean debit)
    {
        this.firePropertyChange("debit", this.debit, debit);
        this.debit = debit;
        this.updateModel();
    }

    /**
     * Flag indicating if the dialog supports remittance {@code Textschluessel}.
     *
     * @return {@code true} if the dialog supports remittance
     * {@code Textschluessel}; {@code false} if not.
     */
    public boolean isRemittance()
    {
        return this.remittance;
    }

    /**
     * Setter for property {@code remittance}.
     *
     * @param remittance {@code true} if the dialog supports remittance
     * {@code Textschluessel}; {@code false} if not.
     */
    public void setRemittance(final boolean remittance)
    {
        this.firePropertyChange("remittance", this.remittance, remittance);
        this.remittance = remittance;
        this.updateModel();
    }

    /**
     * Gets the date used by the instance for searching the currency directory.
     *
     * @return the date used by the instance for searching the currency
     * directory or {@code null}.
     */
    public Date getDate()
    {
        return this.date != null ? this.date : new Date();
    }

    /**
     * Setter for property {@code date}.
     *
     * @param date the date used by the instance for searching the currency
     * directory or {@code null}.
     */
    public void setDate(final Date date)
    {
        this.firePropertyChange("data", this.date, date);
        this.date = date;
        this.updateCurrencies();
    }

    /**
     * Gets the {@code ActionListener} called when the user requests this
     * dialog's operation.
     *
     * @return the {@code ActionListener} called when the user requests this
     * dialog's operation or {@code null}.
     */
    public ActionListener getActionListener()
    {
        return this.actionListener;
    }

    /**
     * Sets the {@code ActionListener} to be called when the user requests this
     * dialog's operation.
     *
     * @param listener the {@code ActionListener} to be called when the user
     * requests this dialog's operation or {@code null}.
     */
    public void setActionListener(final ActionListener listener)
    {
        this.actionListener = listener;
    }

    /**
     * Updates the dialog according to a caught
     * {@code IllegalTransactionException}.
     *
     * @param e the caught {@code IllegalTransactionException} to update the
     * dialog with or {@code null} to reset the dialog from any caught
     * {@code IllegalTransactionException}s.
     */
    public void onIllegalTransaction(final IllegalTransactionException e)
    {
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                amountTextField.setToolTipText(null);
                currencyComboBox.setToolTipText(null);
                descriptionTextField1.setToolTipText(null);
                descriptionTextField2.setToolTipText(null);
                descriptionTextField3.setToolTipText(null);
                descriptionTextField4.setToolTipText(null);
                descriptionTextField5.setToolTipText(null);
                descriptionTextField6.setToolTipText(null);
                descriptionTextField7.setToolTipText(null);
                descriptionTextField8.setToolTipText(null);
                descriptionTextField9.setToolTipText(null);
                descriptionTextField10.setToolTipText(null);
                descriptionTextField11.setToolTipText(null);
                descriptionTextField12.setToolTipText(null);
                descriptionTextField13.setToolTipText(null);
                descriptionTextField14.setToolTipText(null);
                executiveAccountTextField.setToolTipText(null);
                executiveBankTextField.setToolTipText(null);
                executiveExtTextField.setToolTipText(null);
                executiveNameTextField.setToolTipText(null);
                referenceTextField.setToolTipText(null);
                targetAccountTextField.setToolTipText(null);
                targetBankTextField.setToolTipText(null);
                targetExtTextField.setToolTipText(null);
                targetNameTextField.setToolTipText(null);
                textschluesselComboBox.setToolTipText(null);

                for(JComponent label : colors.keySet())
                {
                    resetForeground(label);
                }

                if(e != null)
                {
                    handleIllegalTransaction(amountLabel, amountTextField,
                        e.getMessages(Transaction.PROP_AMOUNT));

                    handleIllegalTransaction(currencyComboBox, currencyComboBox,
                        e.getMessages(Transaction.PROP_CURRENCY));

                    final Message[] descriptions =
                        e.getMessages(Transaction.PROP_DESCRIPTIONS);

                    handleIllegalTransaction(descriptionLabel1,
                        descriptionTextField1, descriptions);

                    handleIllegalTransaction(descriptionLabel2,
                        descriptionTextField2, descriptions);

                    handleIllegalTransaction(descriptionLabel3,
                        descriptionTextField3, descriptions);

                    handleIllegalTransaction(descriptionLabel4,
                        descriptionTextField4, descriptions);

                    handleIllegalTransaction(descriptionLabel5,
                        descriptionTextField5, descriptions);

                    handleIllegalTransaction(descriptionLabel6,
                        descriptionTextField6, descriptions);

                    handleIllegalTransaction(descriptionLabel7,
                        descriptionTextField7, descriptions);

                    handleIllegalTransaction(descriptionLabel8,
                        descriptionTextField8, descriptions);

                    handleIllegalTransaction(descriptionLabel9,
                        descriptionTextField9, descriptions);

                    handleIllegalTransaction(descriptionLabel10,
                        descriptionTextField10, descriptions);

                    handleIllegalTransaction(descriptionLabel11,
                        descriptionTextField11, descriptions);

                    handleIllegalTransaction(descriptionLabel12,
                        descriptionTextField12, descriptions);

                    handleIllegalTransaction(descriptionLabel13,
                        descriptionTextField13, descriptions);

                    handleIllegalTransaction(descriptionLabel14,
                        descriptionTextField14, descriptions);

                    handleIllegalTransaction(executiveAccountLabel,
                        executiveAccountTextField,
                        e.getMessages(Transaction.PROP_EXECUTIVEACCOUNT));

                    handleIllegalTransaction(executiveBankLabel,
                        executiveBankTextField,
                        e.getMessages(Transaction.PROP_EXECUTIVEBANK));

                    handleIllegalTransaction(executiveExtLabel,
                        executiveExtTextField,
                        e.getMessages(Transaction.PROP_EXECUTIVEEXT));

                    handleIllegalTransaction(executiveNameLabel,
                        executiveNameTextField,
                        e.getMessages(Transaction.PROP_EXECUTIVENAME));

                    handleIllegalTransaction(referenceLabel,
                        referenceTextField,
                        e.getMessages(Transaction.PROP_REFERENCE));

                    handleIllegalTransaction(targetAccountLabel,
                        targetAccountTextField,
                        e.getMessages(Transaction.PROP_TARGETACCOUNT));

                    handleIllegalTransaction(targetBankLabel,
                        targetBankTextField,
                        e.getMessages(Transaction.PROP_TARGETBANK));

                    handleIllegalTransaction(targetExtLabel, targetExtTextField,
                        e.getMessages(Transaction.PROP_TARGETEXT));

                    handleIllegalTransaction(targetNameLabel,
                        targetNameTextField,
                        e.getMessages(Transaction.PROP_TARGETNAME));

                    handleIllegalTransaction(textschluesselLabel,
                        textschluesselComboBox,
                        e.getMessages(Transaction.PROP_TYPE));

                    final Message[] msgs = e.getMessages();
                    if(msgs.length > 0)
                    {
                        final StringBuffer buf =
                            new StringBuffer(msgs.length * 150);

                        for(int i = 0; i < msgs.length; i++)
                        {
                            buf.append(msgs[i].getText(Locale.getDefault())).
                                append('\n');

                        }

                        JOptionPane.showMessageDialog(getUserInterface().
                            getParent(), buf.toString(), e.getMessage(),
                            JOptionPane.INFORMATION_MESSAGE);

                    }
                }
            }
        });
    }

    /**
     * Flag indicating if the dialog is locked for any interaction.
     *
     * @return {@code true} if the dialog is locked for any interaction;
     * {@code false} if the dialog is not locked.
     */
    public boolean isLocked()
    {
        return this.locked;
    }

    /**
     * Setter for property {@code locked}.
     *
     * @param locked {@code true} if the dialog should be locked for any
     * interaction; {@code false} to unlock the dialog.
     */
    public void setLocked(final boolean locked)
    {
        this.setCursor(Cursor.getPredefinedCursor(locked ?
            Cursor.WAIT_CURSOR : Cursor.DEFAULT_CURSOR));

        this.amountTextField.setEnabled(!locked);
        this.cancelButton.setEnabled(!locked);
        this.currencyComboBox.setEnabled(!locked);
        this.descriptionTextField1.setEnabled(!locked);
        this.descriptionTextField2.setEnabled(!locked);
        this.descriptionTextField3.setEnabled(!locked);
        this.descriptionTextField4.setEnabled(!locked);
        this.descriptionTextField5.setEnabled(!locked);
        this.descriptionTextField6.setEnabled(!locked);
        this.descriptionTextField7.setEnabled(!locked);
        this.descriptionTextField8.setEnabled(!locked);
        this.descriptionTextField9.setEnabled(!locked);
        this.descriptionTextField10.setEnabled(!locked);
        this.descriptionTextField11.setEnabled(!locked);
        this.descriptionTextField12.setEnabled(!locked);
        this.descriptionTextField13.setEnabled(!locked);
        this.descriptionTextField14.setEnabled(!locked);
        this.executiveAccountTextField.setEnabled(!locked);
        this.executiveBankTextField.setEnabled(!locked);
        this.executiveBankleitzahlButton.setEnabled(!locked);
        this.executiveExtTextField.setEnabled(!locked);
        this.executiveNameTextField.setEnabled(!locked);
        this.okButton.setEnabled(!locked);
        this.referenceTextField.setEnabled(!locked);
        this.targetAccountTextField.setEnabled(!locked);
        this.targetBankTextField.setEnabled(!locked);
        this.targetBankleitzahlButton.setEnabled(!locked);
        this.targetExtTextField.setEnabled(!locked);
        this.targetNameTextField.setEnabled(!locked);
        this.textschluesselComboBox.setEnabled(!locked);

        this.firePropertyChange("locked", this.locked, locked);
        this.locked = locked;
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents()
    {
        okButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();
        transactionPanel = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        executiveNameLabel = new javax.swing.JLabel();
        executiveNameTextField = new AlphaNumericText27TextField();
        executiveExtLabel = new javax.swing.JLabel();
        executiveExtTextField = new AlphaNumericText27TextField();
        executiveBankLabel = new javax.swing.JLabel();
        executiveBankTextField = new BankleitzahlTextField();
        executiveAccountLabel = new javax.swing.JLabel();
        executiveAccountTextField = new KontonummerTextField();
        targetNameLabel = new javax.swing.JLabel();
        targetNameTextField = new AlphaNumericText27TextField();
        targetExtLabel = new javax.swing.JLabel();
        targetExtTextField = new AlphaNumericText27TextField();
        targetBankLabel = new javax.swing.JLabel();
        targetBankTextField = new BankleitzahlTextField();
        targetAccountLabel = new javax.swing.JLabel();
        targetAccountTextField = new KontonummerTextField();
        amountLabel = new javax.swing.JLabel();
        amountTextField = new javax.swing.JTextField();
        executiveBankleitzahlButton = new javax.swing.JButton();
        targetBankleitzahlButton = new javax.swing.JButton();
        currencyComboBox = new javax.swing.JComboBox();
        referenceLabel = new javax.swing.JLabel();
        referenceTextField = new Referenznummer11TextField();
        jPanel2 = new javax.swing.JPanel();
        descriptionTextField1 = new AlphaNumericText27TextField();
        descriptionLabel2 = new javax.swing.JLabel();
        descriptionTextField2 = new AlphaNumericText27TextField();
        descriptionLabel1 = new javax.swing.JLabel();
        descriptionLabel3 = new javax.swing.JLabel();
        descriptionTextField3 = new AlphaNumericText27TextField();
        descriptionLabel4 = new javax.swing.JLabel();
        descriptionTextField4 = new AlphaNumericText27TextField();
        descriptionLabel5 = new javax.swing.JLabel();
        descriptionTextField5 = new AlphaNumericText27TextField();
        descriptionLabel6 = new javax.swing.JLabel();
        descriptionTextField6 = new AlphaNumericText27TextField();
        descriptionLabel7 = new javax.swing.JLabel();
        descriptionTextField7 = new AlphaNumericText27TextField();
        descriptionLabel8 = new javax.swing.JLabel();
        descriptionTextField8 = new AlphaNumericText27TextField();
        descriptionLabel9 = new javax.swing.JLabel();
        descriptionTextField9 = new AlphaNumericText27TextField();
        descriptionLabel10 = new javax.swing.JLabel();
        descriptionTextField10 = new AlphaNumericText27TextField();
        descriptionLabel11 = new javax.swing.JLabel();
        descriptionLabel12 = new javax.swing.JLabel();
        descriptionLabel13 = new javax.swing.JLabel();
        descriptionLabel14 = new javax.swing.JLabel();
        descriptionTextField11 = new AlphaNumericText27TextField();
        descriptionTextField12 = new AlphaNumericText27TextField();
        descriptionTextField13 = new AlphaNumericText27TextField();
        descriptionTextField14 = new AlphaNumericText27TextField();
        textschluesselLabel = new javax.swing.JLabel();
        textschluesselComboBox = new javax.swing.JComboBox();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle(TransactionDialogBundle.getTitleText(Locale.getDefault()));
        setName("Transaction Dialog");
        addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(java.awt.event.WindowEvent evt)
            {
                closeDialog(evt);
            }
        });

        okButton.setText("OK");
        okButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                okButtonActionPerformed(evt);
            }
        });

        cancelButton.setText("Cancel");
        cancelButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                cancelButtonActionPerformed(evt);
            }
        });

        transactionPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(TransactionDialogBundle.getTransactionPanelTitleText(Locale.getDefault())));
        executiveNameLabel.setText(TransactionDialogBundle.getExecutiveNameLabelText(Locale.getDefault()));

        executiveNameTextField.setColumns(AlphaNumericText27.MAX_LENGTH);

        executiveExtLabel.setText(TransactionDialogBundle.getExecutiveName2LabelText(Locale.getDefault()));

        executiveExtTextField.setColumns(AlphaNumericText27.MAX_LENGTH);

        executiveBankLabel.setText(TransactionDialogBundle.getExecutiveBankLabelText(Locale.getDefault()));

        executiveBankTextField.setColumns(Bankleitzahl.MAX_CHARACTERS);

        executiveAccountLabel.setText(TransactionDialogBundle.getExecutiveAccountLabelText(Locale.getDefault()));

        executiveAccountTextField.setColumns(Kontonummer.MAX_CHARACTERS);

        targetNameLabel.setText(TransactionDialogBundle.getTargetNameLabelText(Locale.getDefault()));

        targetNameTextField.setColumns(AlphaNumericText27.MAX_LENGTH);

        targetExtLabel.setText(TransactionDialogBundle.getTargetName2LabelText(Locale.getDefault()));

        targetExtTextField.setColumns(AlphaNumericText27.MAX_LENGTH);

        targetBankLabel.setText(TransactionDialogBundle.getTargetBankLabelText(Locale.getDefault()));

        targetBankTextField.setColumns(Bankleitzahl.MAX_CHARACTERS);

        targetAccountLabel.setText(TransactionDialogBundle.getTargetAccountLabelText(Locale.getDefault()));

        targetAccountTextField.setColumns(Kontonummer.MAX_CHARACTERS);

        amountLabel.setText(TransactionDialogBundle.getAmountLabelText(Locale.getDefault()));

        executiveBankleitzahlButton.setAction(new SearchBankleitzahlAction(this.executiveBankTextField, true));
        executiveBankleitzahlButton.setBorder(null);
        executiveBankleitzahlButton.setBorderPainted(false);
        executiveBankleitzahlButton.setContentAreaFilled(false);
        executiveBankleitzahlButton.setDefaultCapable(false);
        executiveBankleitzahlButton.setMaximumSize(new java.awt.Dimension(16, 16));
        executiveBankleitzahlButton.setMinimumSize(new java.awt.Dimension(16, 16));
        executiveBankleitzahlButton.setPreferredSize(new java.awt.Dimension(16, 16));

        targetBankleitzahlButton.setAction(new SearchBankleitzahlAction(this.targetBankTextField, true));
        targetBankleitzahlButton.setBorder(null);
        targetBankleitzahlButton.setBorderPainted(false);
        targetBankleitzahlButton.setContentAreaFilled(false);
        targetBankleitzahlButton.setDefaultCapable(false);
        targetBankleitzahlButton.setMaximumSize(new java.awt.Dimension(16, 16));
        targetBankleitzahlButton.setMinimumSize(new java.awt.Dimension(16, 16));
        targetBankleitzahlButton.setPreferredSize(new java.awt.Dimension(16, 16));

        currencyComboBox.setRenderer(new CurrencyRenderer());

        referenceLabel.setText(TransactionDialogBundle.getReferenceLabelText(Locale.getDefault()));

        referenceTextField.setColumns(Referenznummer11.MAX_CHARACTERS);

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(referenceLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                    .add(executiveExtLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                    .add(executiveBankLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                    .add(executiveAccountLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                    .add(targetNameLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                    .add(targetExtLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                    .add(targetAccountLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                    .add(targetBankLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                    .add(executiveNameLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE)
                    .add(amountLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 131, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(amountTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 87, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(currencyComboBox, 0, 93, Short.MAX_VALUE))
                    .add(targetAccountTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 186, Short.MAX_VALUE)
                    .add(targetExtTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 186, Short.MAX_VALUE)
                    .add(targetNameTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 186, Short.MAX_VALUE)
                    .add(executiveAccountTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 186, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                        .add(executiveBankTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 164, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(executiveBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(executiveNameTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 186, Short.MAX_VALUE)
                    .add(executiveExtTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 186, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                        .add(targetBankTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 164, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(targetBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(referenceTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 186, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .add(16, 16, 16)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(executiveNameTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(executiveNameLabel))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(executiveExtLabel)
                    .add(executiveExtTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(executiveBankLabel)
                    .add(executiveBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(executiveBankTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(executiveAccountLabel)
                    .add(executiveAccountTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(targetNameLabel)
                    .add(targetNameTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(targetExtLabel)
                    .add(targetExtTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(targetBankLabel)
                    .add(targetBankleitzahlButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(targetBankTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(targetAccountLabel)
                    .add(targetAccountTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(amountLabel)
                    .add(amountTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(currencyComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(referenceLabel)
                    .add(referenceTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(167, Short.MAX_VALUE))
        );

        descriptionTextField1.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel2.setText(TransactionDialogBundle.getDescriptionLabel2Text(Locale.getDefault()));

        descriptionTextField2.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel1.setText(TransactionDialogBundle.getDescriptionLabel1Text(Locale.getDefault()));

        descriptionLabel3.setText(TransactionDialogBundle.getDescriptionLabel3Text(Locale.getDefault()));

        descriptionTextField3.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel4.setText(TransactionDialogBundle.getDescriptionLabel4Text(Locale.getDefault()));

        descriptionTextField4.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel5.setText(TransactionDialogBundle.getDescriptionLabel5Text(Locale.getDefault()));

        descriptionTextField5.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel6.setText(TransactionDialogBundle.getDescriptionLabel6Text(Locale.getDefault()));

        descriptionTextField6.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel7.setText(TransactionDialogBundle.getDescriptionLabel7Text(Locale.getDefault()));

        descriptionTextField7.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel8.setText(TransactionDialogBundle.getDescriptionLabel8Text(Locale.getDefault()));

        descriptionTextField8.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel9.setText(TransactionDialogBundle.getDescriptionLabel9Text(Locale.getDefault()));

        descriptionTextField9.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel10.setText(TransactionDialogBundle.getDescriptionLabel10Text(Locale.getDefault()));

        descriptionTextField10.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionLabel11.setText(TransactionDialogBundle.getDescriptionLabel11Text(Locale.getDefault()));

        descriptionLabel12.setText(TransactionDialogBundle.getDescriptionLabel12Text(Locale.getDefault()));

        descriptionLabel13.setText(TransactionDialogBundle.getDescriptionLabel13Text(Locale.getDefault()));

        descriptionLabel14.setText(TransactionDialogBundle.getDescriptionLabel14Text(Locale.getDefault()));

        descriptionTextField11.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionTextField12.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionTextField13.setColumns(AlphaNumericText27.MAX_LENGTH);

        descriptionTextField14.setColumns(AlphaNumericText27.MAX_LENGTH);

        org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(descriptionLabel3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 158, Short.MAX_VALUE)
                    .add(descriptionLabel4, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 158, Short.MAX_VALUE)
                    .add(descriptionLabel5, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel6, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel7, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 128, Short.MAX_VALUE)
                    .add(descriptionLabel8, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel9, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel10, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel11, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel12, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel13, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(descriptionLabel14, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(descriptionTextField14, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(descriptionTextField13, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(descriptionTextField12, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(descriptionTextField11, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(descriptionTextField10, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(descriptionTextField9, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(descriptionTextField8, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, descriptionTextField7, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, descriptionTextField6, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, descriptionTextField5, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, descriptionTextField4, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, descriptionTextField3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, descriptionTextField2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel2Layout.createSequentialGroup()
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(descriptionTextField1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 159, Short.MAX_VALUE)))
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionLabel1)
                    .add(descriptionTextField1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionTextField2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(descriptionLabel2))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionLabel3)
                    .add(descriptionTextField3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionLabel4)
                    .add(descriptionTextField4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionLabel5)
                    .add(descriptionTextField5, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionLabel6)
                    .add(descriptionTextField6, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionLabel7)
                    .add(descriptionTextField7, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionTextField8, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(descriptionLabel8))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionTextField9, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(descriptionLabel9))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionTextField10, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(descriptionLabel10))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionTextField11, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(descriptionLabel11))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionTextField12, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(descriptionLabel12))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionTextField13, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(descriptionLabel13))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(descriptionTextField14, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(descriptionLabel14))
                .addContainerGap(76, Short.MAX_VALUE))
        );

        textschluesselLabel.setText(TransactionDialogBundle.getTextschluesselLabelText(Locale.getDefault()));

        textschluesselComboBox.setRenderer(new TextschluesselRenderer());

        org.jdesktop.layout.GroupLayout transactionPanelLayout = new org.jdesktop.layout.GroupLayout(transactionPanel);
        transactionPanel.setLayout(transactionPanelLayout);
        transactionPanelLayout.setHorizontalGroup(
            transactionPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(transactionPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(transactionPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, transactionPanelLayout.createSequentialGroup()
                        .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(transactionPanelLayout.createSequentialGroup()
                        .add(textschluesselLabel)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(textschluesselComboBox, 0, 616, Short.MAX_VALUE)))
                .addContainerGap())
        );
        transactionPanelLayout.setVerticalGroup(
            transactionPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(transactionPanelLayout.createSequentialGroup()
                .add(transactionPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(textschluesselLabel)
                    .add(textschluesselComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(transactionPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(jPanel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(okButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 67, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(cancelButton))
                    .add(transactionPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );

        layout.linkSize(new java.awt.Component[] {cancelButton, okButton}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(transactionPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(cancelButton)
                    .add(okButton))
                .addContainerGap())
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void okButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_okButtonActionPerformed
    {
        if(this.getActionListener() != null)
        {
            this.getActionListener().actionPerformed(evt);
        }
    }//GEN-LAST:event_okButtonActionPerformed

    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_cancelButtonActionPerformed
    {
        this.doClose(RET_CANCEL);
    }//GEN-LAST:event_cancelButtonActionPerformed

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt)//GEN-FIRST:event_closeDialog
    {
        if(!this.isLocked())
        {
            this.doClose(RET_CANCEL);
        }
    }//GEN-LAST:event_closeDialog

    private void doClose(int retStatus)
    {
        this.returnStatus = retStatus;
        this.setVisible(false);
        this.dispose();
    }

    private void setForeground(final JComponent label, final Color color)
    {
        if(!this.colors.containsKey(label))
        {
            this.colors.put(label, label.getForeground());
        }
        label.setForeground(color);
    }

    private void resetForeground(final JComponent label)
    {
        if(this.colors.containsKey(label))
        {
            label.setForeground(this.colors.get(label));
        }
    }

    private void handleIllegalTransaction(final JComponent label,
        final JComponent comp, final Message[] msgs)
    {
        if(msgs.length > 0)
        {
            final StringBuffer tooltip = new StringBuffer(500).append("<html>");

            for(int i = 0; i < msgs.length; i++)
            {
                tooltip.append(msgs[i].getText(Locale.getDefault())).
                    append("<br>");

            }

            comp.setToolTipText(tooltip.append("</html>").toString());
            this.setForeground(label, Color.RED);
        }
        else
        {
            comp.setToolTipText(null);
            this.resetForeground(label);
        }
    }

    private JTextComponent getDescriptionComponent(final int index)
    {
        JTextComponent comp;

        switch(index)
        {
            case 1: comp = this.descriptionTextField1; break;
            case 2: comp = this.descriptionTextField2; break;
            case 3: comp = this.descriptionTextField3; break;
            case 4: comp = this.descriptionTextField4; break;
            case 5: comp = this.descriptionTextField5; break;
            case 6: comp = this.descriptionTextField6; break;
            case 7: comp = this.descriptionTextField7; break;
            case 8: comp = this.descriptionTextField8; break;
            case 9: comp = this.descriptionTextField9; break;
            case 10: comp = this.descriptionTextField10; break;
            case 11: comp = this.descriptionTextField11; break;
            case 12: comp = this.descriptionTextField12; break;
            case 13: comp = this.descriptionTextField13; break;
            case 14: comp = this.descriptionTextField14; break;
            default:
                throw new IllegalArgumentException(Integer.toString(index));

        }

        return comp;
    }

    private void updateModel()
    {
        final Textschluessel[] keys = getTextschluesselVerzeichnis().
            search(isDebit(), isRemittance());

        Arrays.sort(keys, new Comparator()
        {
            public int compare(final Object o1, final Object o2)
            {
                assert o1 instanceof Textschluessel;
                assert o2 instanceof Textschluessel;

                final Textschluessel t1 = (Textschluessel) o1;
                final Textschluessel t2 = (Textschluessel) o2;

                int ret = t1.getKey() > t2.getKey() ? 1 :
                    t1.getKey() < t2.getKey() ? -1 : 0;

                if(ret == 0)
                {
                    ret = t1.getExtension() > t2.getExtension() ? 1:
                        t1.getExtension() < t2.getExtension() ? -1 : 0;

                }

                return ret;
            }
        });

        final Object prevSelection =
            textschluesselComboBox.getSelectedItem();

        textschluesselComboBox.removeAllItems();
        for(int i = 0; i < keys.length; i++)
        {
            textschluesselComboBox.addItem(keys[i]);
        }

        textschluesselComboBox.setSelectedItem(prevSelection);
    }

    private void updateCurrencies()
    {
        final Currency[] currencies = getCurrencyDirectory().
            getDtausCurrencies(this.getDate());

        Arrays.sort(currencies, new Comparator()
        {
            public int compare(final Object o1, final Object o2)
            {
                assert o1 instanceof Currency;
                assert o2 instanceof Currency;

                return ((Currency) o1).getCurrencyCode().
                    compareTo(((Currency) o2).getCurrencyCode());

            }
        });

        final Object prevSelection = currencyComboBox.getSelectedItem();
        currencyComboBox.removeAllItems();
        for(int i = 0; i < currencies.length; i++)
        {
            currencyComboBox.addItem(currencies[i]);
        }

        currencyComboBox.setSelectedItem(prevSelection);
    }

    private boolean isBankcodeValid(final Bankleitzahl bankcode)
    {
        boolean valid = bankcode != null;

        if(valid)
        {
            valid = this.getBankleitzahlenVerzeichnis().
                getHeadOffice(bankcode) != null;

        }

        return valid;
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel amountLabel;
    private javax.swing.JTextField amountTextField;
    private javax.swing.JButton cancelButton;
    private javax.swing.JComboBox currencyComboBox;
    private javax.swing.JLabel descriptionLabel1;
    private javax.swing.JLabel descriptionLabel10;
    private javax.swing.JLabel descriptionLabel11;
    private javax.swing.JLabel descriptionLabel12;
    private javax.swing.JLabel descriptionLabel13;
    private javax.swing.JLabel descriptionLabel14;
    private javax.swing.JLabel descriptionLabel2;
    private javax.swing.JLabel descriptionLabel3;
    private javax.swing.JLabel descriptionLabel4;
    private javax.swing.JLabel descriptionLabel5;
    private javax.swing.JLabel descriptionLabel6;
    private javax.swing.JLabel descriptionLabel7;
    private javax.swing.JLabel descriptionLabel8;
    private javax.swing.JLabel descriptionLabel9;
    private javax.swing.JTextField descriptionTextField1;
    private javax.swing.JTextField descriptionTextField10;
    private javax.swing.JTextField descriptionTextField11;
    private javax.swing.JTextField descriptionTextField12;
    private javax.swing.JTextField descriptionTextField13;
    private javax.swing.JTextField descriptionTextField14;
    private javax.swing.JTextField descriptionTextField2;
    private javax.swing.JTextField descriptionTextField3;
    private javax.swing.JTextField descriptionTextField4;
    private javax.swing.JTextField descriptionTextField5;
    private javax.swing.JTextField descriptionTextField6;
    private javax.swing.JTextField descriptionTextField7;
    private javax.swing.JTextField descriptionTextField8;
    private javax.swing.JTextField descriptionTextField9;
    private javax.swing.JLabel executiveAccountLabel;
    private javax.swing.JTextField executiveAccountTextField;
    private javax.swing.JLabel executiveBankLabel;
    private javax.swing.JTextField executiveBankTextField;
    private javax.swing.JButton executiveBankleitzahlButton;
    private javax.swing.JLabel executiveExtLabel;
    private javax.swing.JTextField executiveExtTextField;
    private javax.swing.JLabel executiveNameLabel;
    private javax.swing.JTextField executiveNameTextField;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JButton okButton;
    private javax.swing.JLabel referenceLabel;
    private javax.swing.JTextField referenceTextField;
    private javax.swing.JLabel targetAccountLabel;
    private javax.swing.JTextField targetAccountTextField;
    private javax.swing.JLabel targetBankLabel;
    private javax.swing.JTextField targetBankTextField;
    private javax.swing.JButton targetBankleitzahlButton;
    private javax.swing.JLabel targetExtLabel;
    private javax.swing.JTextField targetExtTextField;
    private javax.swing.JLabel targetNameLabel;
    private javax.swing.JTextField targetNameTextField;
    private javax.swing.JComboBox textschluesselComboBox;
    private javax.swing.JLabel textschluesselLabel;
    private javax.swing.JPanel transactionPanel;
    // End of variables declaration//GEN-END:variables

    /**
     * Return status code.
     * @serial
     */
    private int returnStatus = RET_CANCEL;

    /**
     * Flag indicating if the dialog supports debit {@code Textschluessel}.
     * @serial
     */
    private boolean debit;

    /**
     * Flag indicating if the dialog supports remittance {@code Textschluessel}.
     * @serial
     */
    private boolean remittance;

    /**
     * Date to be used for calculating the available currencies.
     * @serial
     */
    private Date date;

    /**
     * Caches label colors.
     * @serial
     */
    private Map<JComponent, Color> colors = new HashMap(50);

    /**
     * Flag indicating that the user interface is locked.
     * @serial
     */
    private boolean locked;

    /**
     * {@code ActionListener} called when the user requests the dialog's
     * operation.
     */
    private transient ActionListener actionListener;

    //-------------------------------------------------------TransactionDialog--

}
