/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.editor;

import java.io.File;
import java.io.IOException;
import java.util.Locale;
import org.jdtaus.banking.dtaus.PhysicalFile;
import org.jdtaus.core.text.Message;

/**
 * Generates a {@code PhysicalFile}.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: PhysicalFileGenerator.java 3512 2007-05-16 05:57:21Z schulte2005 $
 */
public interface PhysicalFileGenerator
{
    //--PhysicalFileGenerator---------------------------------------------------

    /**
     * Writes a {@code PhysicalFile} to the given file using the given data.
     *
     * @param file the file to write a {@code PhysicalFile} to.
     * @param command the data to use for the written file.
     *
     * @return the generated file or {@code null} if generation got cancelled.
     *
     * @throws NullPointerException if either {@code file} or {@code command}
     * is {@code null}.
     * @throws IllegalGeneratorTaskException if {@code command} holds illegal
     * values.
     * @throws IOException if writing fails.
     */
    PhysicalFile generate(File file, GeneratorTask command) throws IOException;

    //---------------------------------------------------PhysicalFileGenerator--
}
